# -*- coding: utf-8 -*-

"""Module of methods for visualizing visibility data.  Routines include:
  * plots of visibility amplitude as a function of uv radius
  * plots of phase for specific baselines as a function of time
  * plots of closure phase for three antennae as a function of time

In addition, most of the functions have the ability to compare the observed 
data with simulations.  The data dictionaries referred to in the functions 
are in the formate as the data dictionaries created by :mod:`lsl.sim.vis`.

.. seealso::
   :mod:`lsl.sim.vis`
"""

import math
import numpy as n

import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter

__version__ = '0.1'
__revision__ = '$ Revision: 15 $'
__all__ = ['argument', 'unwrap', 'unmaskCompressedData', 'plotVisibilities', 'plotPhases', 'fitPhases', 'plotClosure', '__version__', '__revision__', '__all__']


def argument(data):
	"""Return the argument (phase) of a complex number.  This function is 
	a 'short cut' for the numpy.angle function."""
	
	return n.angle(data)


def unwrap(theta):
	"""Given a numpy array of phases (complex number arguments), unwrap the 
	phases by looking for +/- pi jumps in the data.  Return an unwrapped 
	numpy array of the same length."""

	output = 1.0*theta
	for i in range(1, len(theta), 1):
		if output[i] - output[i-1] >= n.pi:
			output[i:] = output[i:] - 2*n.pi
		elif output[i] - output[i-1] <= -n.pi:
			output[i:] = output[i:] + 2*n.pi
	return output


def unmaskCompressedData(data, mask):
	"""Given a numpy array that has been compressed and the mask used to 
	compress it, rebuild the original full-sized array.  Values that were 
	previously masked are replaced with NaNs."""

	origData = n.zeros(len(mask), dtype=data.dtype)
	
	maskCount = 0
	dataCount = 0
	for maskValue in mask:
		if maskValue == 0:
			origData[maskCount] = data[dataCount]
			dataCount = dataCount + 1
		else:
			origData[maskCount] = n.nan
		maskCount = maskCount + 1

	return origData


def plotVisibilities(dataDict,  pol='yy', chan=None, jd=None, simDict=None, SaveFig=False):
	"""Given a data dictionary from readUVData, create a plot of visibility
	amplitude as a function of uv radius.  Alternately, a Julian Date can be 
	specified and only data matched that date are plotted.  If a dictionary of 
	simulated data are also supplied via the simDict keyword, a two-panel plot
	is created comparing the real and simulated data."""
	
	fig = plt.figure()
	if simDict is not None:
		ax1 = fig.add_subplot(2, 1, 1)
		ax2 = fig.add_subplot(2, 1, 2)
	else:
		ax1 = fig.add_subplot(1, 1, 1)

	# Calculate the uv radius, normalize the amplitudes, and plot.  Each
	# baseline is done all at once so they all have the same colors
	for t, uvw, data in zip(dataDict['jd'][pol], dataDict['uvw'][pol], dataDict['vis'][pol]):
		if jd is not None:
			if t != jd:
				continue
		if chan is None:
			uvDist = n.sqrt( uvw[0,:]**2 + uvw[1,:]**2)
			visAmp = n.abs(data)
		else:
			uvDist = n.array([n.sqrt( uvw[0,chan]**2 + uvw[1,chan]**2)])
			visAmp = n.array(n.abs(data[chan]))
		
		ax1.plot(uvDist, visAmp, marker='o', linestyle=' ', alpha=0.50)
		ax1.semilogy()

	if simDict is not None:
		for t, uvw, data in zip(simDict['jd'][pol], simDict['uvw'][pol], simDict['vis'][pol]):
			if jd is not None:
				if t != jd:
					continue
			if chan is None:
				uvDist = n.sqrt( uvw[0,:]**2 + uvw[1,:]**2)
				visAmp = n.abs(data)
			else:
				uvDist = n.array([n.sqrt( uvw[0,chan]**2 + uvw[1,chan]**2)])
				visAmp = n.array(n.abs(data[chan]))
			
			ax2.plot(uvDist, visAmp, marker='o', linestyle=' ', alpha=0.50)
			ax2.semilogy()

	# Set axis title and labels
	ax1.set_title('Pol.: %s' % pol)
	ax1.set_xlabel('$uv$ Radius [$\lambda$]')
	ax1.set_ylabel('Amplitude')

	if simDict is not None:
		ax2.set_title('Pol.: %s (Sim.)' % pol)
		ax2.set_xlabel('$uv$ Radius [$\lambda$]')
		ax2.set_ylabel('Amplitude')

	plt.show()

	if SaveFig:
		fig.savefig('visibilities.png')


def plotPhases(dataDict, baselines=[0], pol='yy', jd=None, stands=None, simDict=None, Unwrap=False, SaveFig=False):
	"""Given a data dictionary from readUVData, create a plot of visibility
	phase as a function of frequency.  If a dictionary of simulated data are 
	also supplied via the simDict keyword, a two-panel plot is created comparing 
	the real and simulated data."""

	fig = plt.figure()
	if simDict is not None:
		ax1 = fig.add_subplot(3, 1, 1)
		ax2 = fig.add_subplot(3, 1, 2)
		ax3 = fig.add_subplot(3, 1, 3)
	else:
		ax1 = fig.add_subplot(1, 1, 1)

	# Loop over baselines since we care about only certain baselines
	for baseline in baselines:
		# Load in the frequency and the data
		freq = n.ma.array(dataDict['freq'], mask=dataDict['msk'][pol][baseline])
		data = dataDict['vis'][pol][baseline]

		# Calculate the phase from the visibility data and generate a label.  If
		# the stands keyword has been used in the call, the label contains actual 
		# stand numbers.
		phs = argument(dataDict['vis'][pol][baseline])
		if Unwrap:
			phs = unwrap(phs)
		if stands is not None:
			i,j = dataDict['bls'][pol][baseline]
			lbl = '%i-%i' % (stands[i], stands[j])
		else:
			lbl = '%i-%i' % dataDict['bls'][pol][baseline]
		
		# Plot depending on if the data are masked or not
		if phs.shape[0] != dataDict['msk'][pol][baseline].shape[0]:
			obsLine = ax1.plot(freq.compressed()/1e6, phs, marker='o', alpha=0.50, label=lbl)
		else:
			obsLine = ax1.plot(freq/1e6, phs, marker='o', alpha=0.50, label=lbl)

		plotColor = obsLine[-1].get_color()
		if simDict is not None:
			# Load in the frequency and the data
			freq = n.ma.array(simDict['freq'], mask=simDict['msk'][pol][baseline])
			sdata = simDict['vis'][pol][baseline]

			# Calculate the phase from the visibility data and generate a label.  If
			# the stands keyword has been used in the call, the label contains actual 
			# stand numbers.
			phs = argument(sdata)
			if Unwrap:
				phs = unwrap(phs)
			if stands is not None:
				i,j = dataDict['bls'][pol][baseline]
				lbl = '%i-%i (S)' % (stands[i], stands[j])
				lbl2 = '%i-%i (D)' % (stands[i], stands[j])
			else:
				lbl = '%i-%i (S)' % dataDict['bls'][pol][baseline]
				lbl2 = '%i-%i (D)' % dataDict['bls'][pol][baseline]
			
			rdata = data / sdata
			rphs = argument(rdata)
			if Unwrap:
				rphs = unwrap(rphs)

			# Plot depending on if the data are masked or not
			if phs.shape[0] != dataDict['msk'][pol][baseline].shape[0]:
				ax2.plot(freq.compressed()/1e6, phs, marker='^', alpha=0.50, label=lbl, color=plotColor)
				ax3.plot(freq.compressed()/1e6, rphs, marker='x', alpha=0.50, label=lbl2, color=plotColor)
			else:
				ax2.plot(freq/1e6, phs, marker='^', alpha=0.50, label=lbl, color=plotColor)
				ax3.plot(freq/1e6, rphs, marker='x', alpha=0.50, label=lbl2, color=plotColor)

			if Unwrap:
				delayCoeffs = n.polyfit(freq, rphs, 1)
				delayFit = n.polyval(delayCoeffs, freq)
				delay = delayCoeffs[0] / 2.0 / n.pi * 1e9
				ax3.plot(freq/1e6, delayFit, linestyle='-', label="Delay: %.2f ns" % delay, color=plotColor)

	if not Unwrap:
		ax1.set_yticks([-math.pi, -math.pi/2, 0, math.pi/2, math.pi])
		ax1.set_yticklabels(['-$\pi$', '-$\pi$/2', '0', '$\pi$/2', '$\pi$'])

	ax1.set_title('Pol.: %s' % pol)
	ax1.set_xlabel('Frequency [MHz]')
	ax1.set_ylabel('Phase [rad]')
		
	ax1.legend(loc=0)

	if simDict is not None:
		if not Unwrap:
			ax2.set_ylim([-math.pi, math.pi])
			ax2.set_yticks([-math.pi, -math.pi/2, 0, math.pi/2, math.pi])
			ax2.set_yticklabels(['-$\pi$', '-$\pi$/2', '0', '$\pi$/2', '$\pi$'])

		ax2.set_title('Pol.: %s (Sim.)' % pol)
		ax2.set_xlabel('Frequency [MHz]')
		ax2.set_ylabel('Phase [rad]')
			
		if not Unwrap:
			ax3.set_ylim([-math.pi, math.pi])
			ax3.set_yticks([-math.pi, -math.pi/2, 0, math.pi/2, math.pi])
			ax3.set_yticklabels(['-$\pi$', '-$\pi$/2', '0', '$\pi$/2', '$\pi$'])

		ax3.set_title('Pol.: %s (Sim.)' % pol)
		ax3.set_xlabel('Frequency [MHz]')
		ax3.set_ylabel('Phase Diff. [rad]')

		ax2.legend(loc=0)
		ax3.legend(loc=0)

	plt.show()

	if SaveFig:
		fig.savefig('phases.png')


def fitPhases(dataDict, simDict, baseline=0, pol='yy', minCorr=0.90, returnDispersion=False, stands=None, NoPlot=False):
	"""Given a data dictionary from readUVData, fit delays by examining the
	change of the visiblility phases with frequency.  This routine examines
	each baseline of interest and looks for flat regions that are 3 MHz wide 
	and have R correlation values greater than 'minCorr'.  In theses flat 
	regions, a line is fit to the data and average slope found from the fits
	yields a delay.  The delays are returned as a numpy array with values in
	ns.  If the returnDispersion keyword is set, a numpy array with the 
	standard deviation of the fits is also returned."""

	import pylab

	# Build a list of unique Julian dates present in the data
	junk = {}
	for dt in dataDict['jd'][pol]:
		junk[dt] = 1
	jds = sorted(junk.keys())
	nJD = len(jds)
	del(junk)

	# Number of baselines in the dataset
	nBL = len(dataDict['vis'][pol]) / nJD
	refBaseline = baseline % nBL

	# Setup the output figure
	if not NoPlot:
		fig = pylab.figure()
		ax1 = fig.add_subplot(2, 1, 1)
		ax2 = fig.add_subplot(2, 1, 2)

	# Loop over baselines
	allDelay = []
	baseDelay = []
	for baseline in range(refBaseline, len(dataDict['vis'][pol]), nBL):
		# Load the frequency and removed flagged values
		freq = n.ma.array(dataDict['freq'], mask=dataDict['msk'][pol][baseline])
		freq = freq.compressed()

		# Load the visibility data and calculate the phase
		phs = unwrap(argument(dataDict['vis'][pol][baseline] / simDict['vis'][pol][baseline]))

		# Plot
		if not NoPlot:
			obsLine = ax1.plot(freq/1e6, phs, marker='x')
			plotColor = obsLine[-1].get_color()

		# Fit the delay of the unwrapped phase difference
		delayCoeffs = n.polyfit(freq, phs, 1)
		delayFit = n.polyval(delayCoeffs, freq)
		delay = delayCoeffs[0] / 2.0 / n.pi * 1e9
		R = n.corrcoef(freq, phs)[0,1]

		allDelay.append( delay )
		if n.abs(R) >= minCorr:
			baseDelay.append( delay )
			if not NoPlot:
				ax1.plot(freq/1e6, delayFit, color=plotColor)

	# Convert baseDelay to a numpy array and remove bad fits characterized by a NaN
	allDelay = n.array(allDelay)
	allDelay = allDelay.compress(n.isfinite(allDelay))
	baseDelay = n.array(baseDelay)
	baseDelay = baseDelay.compress(n.isfinite(baseDelay))
	print refBaseline, len(allDelay), allDelay.mean(), allDelay.std()
	print refBaseline, len(baseDelay), baseDelay.mean(), baseDelay.std()

	delays = baseDelay.mean()
	disprs = baseDelay.std()
	
	if not NoPlot:
		dataRange = baseDelay.max() - baseDelay.min()
		nBins = int(round(n.abs(dataRange)/15.0))
		ax2.hist(allDelay, bins=nBins, range=[allDelay.min(), allDelay.max()], label='All')
		ax2.hist(baseDelay, bins=nBins, range=[allDelay.min(), allDelay.max()], label='R$\geq$%.3f' % minCorr)

		title = 'Delays for Baseline #%i' % refBaseline
		if stands is not None:
			i, j = dataDict['bls'][pol][refBaseline]
			title = "%s (%i-%i)" % (title, stands[i], stands[j])
		ax1.set_title(title)
		ax1.set_xlabel('Frequency [MHz]')
		ax1.set_ylabel('$\Delta$ Phase [rad]')
		ax2.set_xlabel('Delay [ns]')
		ax2.set_ylabel('Number of Samples')
		ax2.legend(loc=0)

		pylab.show()

	if returnDispersion:
		return (delays, disprs)
	else:
		return delays


def plotClosure(dataDict, pol='yy', antennas=[1, 2, 3], individualPhases=False, stands=None, SaveFig=False):
	"""Given a data dictionary from readUVData, plot the closure phase of three 
	antennae as a function of time.  If the individualPhases keyword is set, also 
	plot the phases over time of the three antennae that comprise the closure 
	triplet."""

	fig = plt.figure()
	if not individualPhases:
		ax1 = fig.add_subplot(1, 1, 1)
	else:
		ax1 = fig.add_subplot(2, 1, 1)
		ax2 = fig.add_subplot(2, 3, 4)
		ax3 = fig.add_subplot(2, 3, 5)
		ax4 = fig.add_subplot(2, 3, 6)

	# Build a list of unique Julian dates present in the data
	junk = {}
	for dt in dataDict['jd'][pol]:
		junk[dt] = 1
	jds = sorted(junk.keys())
	print jds
	del(junk)

	# Loop over the JDs to build up the closure phase as a function of time
	for jd in jds:
		print "Starting on JD %.5f" % jd
		# Initialize closerPhase to 0 for all frequencies
		trispectrum = n.ones_like(dataDict['freq'])
		individPhase = n.zeros((3,len(dataDict['freq'])))
		closureCount = 0
		
		# Loop over the data
		for (i,j),vis,msk,t in zip(dataDict['bls'][pol], dataDict['vis'][pol], dataDict['msk'][pol], dataDict['jd'][pol]):
			if t < jd:
				continue
			if t > jd:
				break

			# If we come across antennae that are part of the closure triplet, get their
			# phases.  Note:  one pair in the triplet needs it Hermitian conjugate to 
			# to properly close the phase.  This pair is taken to be the baseline formed
			# by the first and last antennae specified.
			if i in antennas and j in antennas:
				if i == antennas[0] and j == antennas[2]:
					temp = j
					j = i
					i = temp
					vis = vis.conj()
				vis = unmaskCompressedData(vis, msk)

				if closureCount == 0:
					trispectrum = vis
				else:
					trispectrum *= vis
				if i == antennas[0]:
					individPhase[0,:] = argument(vis)
				elif i == antennas[1]:
					individPhase[1,:] = argument(vis)
				else:
					individPhase[2,:] = argument(vis)
				closureCount += 1

			# If clouseCount indicates that we have added all 3 phases together, plot the
			# result
			if closureCount == 3:
				toUse = [len(dataDict['freq'])/4, len(dataDict['freq'])/2, 3*len(dataDict['freq'])/4]

				closurePhase = argument(trispectrum)

				time = (n.ones_like(dataDict['freq'])*t - jds[0])
				c = ax1.scatter(time[toUse]*24.0*60.0, closurePhase[toUse]*180.0/n.pi, c=dataDict['freq'][toUse])
				
				if individualPhases:
					bad = n.where( individPhase > n.pi )
					if len(bad[0]) > 0:
						individPhase -= (2*n.pi)

					ax2.scatter(time[toUse]*24.0*60.0, individPhase[0,toUse]*180.0/n.pi, c=dataDict['freq'][toUse])
					ax3.scatter(time[toUse]*24.0*60.0, individPhase[1,toUse]*180.0/n.pi, c=dataDict['freq'][toUse])
					ax4.scatter(time[toUse]*24.0*60.0, individPhase[2,toUse]*180.0/n.pi, c=dataDict['freq'][toUse])
				break

	if stands is None:
		title = 'Closure Phase for %i-%i-%i' % (antennas[0],antennas[1],antennas[2])
	else:
		title = 'Closure Phase for %i-%i-%i' % (stands[antennas[0]],stands[antennas[1]],stands[antennas[2]])
	ax1.set_title(title)
	ax1.set_xlabel('Time from First Capture [min]')
	ax1.set_ylabel('Closure Phase [deg]')
	step = len(dataDict['freq'])/10
	b = fig.colorbar(c, ax=ax1, ticks=dataDict['freq'][0::step])
	b.ax.set_yticklabels(["%.2f MHz" % (1e-6*f) for f in (dataDict['freq'][0::step])])

	ax1.set_ylim([-180, 180])
	if individualPhases:
		ax2.set_ylabel('Phase [deg]')
		ax2.set_ylim([-180, 180])
		ax3.set_ylim([-180, 180])
		ax3.yaxis.set_major_formatter( NullFormatter() )
		ax4.set_ylim([-180, 180])
		ax4.yaxis.set_major_formatter( NullFormatter() )

	plt.show()

	if SaveFig:
		fig.savefig('closure.png')


def plotWaterfall(dataDict, baseline=21, pol='yy', stands=None, simDict=None, SaveFig=False):
	"""Create a waterfall diagram (visibility amplitude as a function of time and 
	frequency) for a particular baseline.  If simulated data are also provided, 
	plot those on a seperate axis."""
	
	fig = plt.figure()
	if simDict is None:
		ax1 = fig.add_subplot(1, 1, 1)
	else:
		ax1 = fig.add_subplot(2, 1, 1)
		ax2 = fig.add_subplot(2, 1, 2)

	# Number of channels in the data set
	nChan = len(dataDict['freq'])

	# Build a list of unique Julian dates present in the data
	junk = {}
	for dt in dataDict['jd'][pol]:
		junk[dt] = 1
	jds = sorted(junk.keys())
	nJD = len(jds)
	del(junk)

	# Number of baselines in the dataset
	nBL = len(dataDict['vis'][pol]) / nJD
	baseline = baseline % nBL

	# Create the waterfall diagram for the data
	wfd = n.zeros((nJD,nChan))
	for i in range(nJD):
		if dataDict['isMasked']:
			wfd[i,:] = unmaskCompressedData(n.abs( dataDict['vis'][pol][baseline+nBL*i] ), dataDict['msk'][pol][baseline+nBL*i])
		else:
			wfd[i,:] = n.abs( dataDict['vis'][pol][baseline+nBL*i] )
	
	# Do the same for the simulated data if it is provided
	if simDict is not None:
		swfd = n.zeros((nJD,nChan))
		for i in range(nJD):
			if simDict['isMasked']:
				swfd[i,:] = unmaskCompressedData(n.abs( simDict['vis'][pol][baseline+nBL*i] ), simDict['msk'][pol][baseline+nBL*i])
			else:
				swfd[i,:] = n.abs( simDict['vis'][pol][baseline+nBL*i] )

	# Determine intelligent clipping levels
	hist, bins = n.histogram(wfd.compress(n.isfinite(wfd.ravel())), bins=1000)
	cdf = 1.0*hist.cumsum() / hist.sum()
	lowerClip = (n.where( cdf >= 0.05, bins[1:], n.zeros_like(bins[1:])+1e12)).min()
	upperClip = (n.where( cdf <= 0.90, bins[1:], n.zeros_like(bins[1:])-1e12)).max()
	print lowerClip, upperClip

	# Plot it all up
	ax1.imshow(wfd, origin='lower', vmin=lowerClip, vmax=upperClip, 
				extent=(dataDict['freq'][0]/1e6, dataDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))
	if simDict is not None:
		# Determine intelligent clipping levels
		hist, bins = n.histogram(swfd.compress(n.isfinite(swfd.ravel())), bins=1000)
		cdf = 1.0*hist.cumsum() / hist.sum()
		lowerClip = (n.where( cdf >= 0.05, bins[1:], n.zeros_like(bins[1:])+1e12)).min()
		upperClip = (n.where( cdf <= 0.95, bins[1:], n.zeros_like(bins[1:])-1e12)).max()
		print lowerClip, upperClip

		ax2.imshow(swfd, origin='lower', vmin=lowerClip, vmax=upperClip, 
					extent=(simDict['freq'][0]/1e6, simDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))

	if stands is None:
		title = 'Visibility Amplitudes for %i-%i' % dataDict['bls'][pol][baseline]
	else:
		i,j = dataDict['bls'][pol][baseline]
		title = 'Visibility Amplitudes for %i-%i' % (stands[i], stands[j])
	ax1.set_title(title)	
	ax1.set_xlabel('Frequency [MHz]')
	ax1.set_ylabel('Time [JD-%.4f]' % jds[0])
	ax1.axis('auto')

	if simDict is not None:
		if stands is None:
			title = 'Visibility Amplitudes for %i-%i (Sim.)' % simDict['bls'][pol][baseline]
		else:
			i,j = simDict['bls'][pol][baseline]
			title = 'Visibility Amplitudes for %i-%i (Sim.)' % (stands[i], stands[j])
		ax2.set_title(title)	
		ax2.set_xlabel('Frequency [MHz]')
		ax2.set_ylabel('Time [JD-%.4f]' % jds[0])
		ax2.axis('auto')

	plt.show()

	if SaveFig:
		fig.savefig('waterfall.png')


def matchAmps(dataDict, baseline=21, pol='yy', stands=None, simDict=None, SaveFig=False):
	"""Create a waterfall diagram (visibility amplitude as a function of time and 
	frequency) for a particular baseline.  If simulated data are also provided, 
	plot those on a seperate axis."""
	
	fig = plt.figure()
	if simDict is None:
		ax1 = fig.add_subplot(1, 1, 1)
	else:
		ax1 = fig.add_subplot(2, 1, 1)
		ax2 = fig.add_subplot(2, 1, 2)

	# Number of channels in the data set
	nChan = len(dataDict['freq'])

	# Build a list of unique Julian dates present in the data
	junk = {}
	for dt in dataDict['jd'][pol]:
		junk[dt] = 1
	jds = sorted(junk.keys())
	nJD = len(jds)
	del(junk)

	# Number of baselines in the dataset
	nBL = len(dataDict['vis'][pol]) / nJD
	baseline = baseline % nBL

	# Create the waterfall diagram for the data
	wfd = n.zeros((nJD,nChan))
	for i in range(nJD):
		if dataDict['isMasked']:
			wfd[i,:] = unmaskCompressedData(n.abs( dataDict['vis'][pol][baseline+nBL*i] ), dataDict['msk'][pol][baseline+nBL*i])
		else:
			wfd[i,:] = n.abs( dataDict['vis'][pol][baseline+nBL*i] )
	

	bestMatch = -1
	bestChi2 = 1e90

	for b in range(nBL):
		# Do the same for the simulated data if it is provided
		if simDict is not None:
			swfd = n.zeros((nJD,nChan))
			for i in range(nJD):
				if simDict['isMasked']:
					swfd[i,:] = unmaskCompressedData(n.abs( simDict['vis'][pol][b+nBL*i] ), simDict['msk'][pol][b+nBL*i])
				else:
					swfd[i,:] = n.abs( simDict['vis'][pol][b+nBL*i] )

		Chi2 = ((wfd - swfd*wfd.std()/swfd.std())**2).sum()
		if Chi2 < bestChi2:
			print 'New best match is %i @ %f' % (b, Chi2)
			bestMatch = b
			bestChi2 = Chi2

	# Do the same for the simulated data if it is provided
	if simDict is not None:
		swfd = n.zeros((nJD,nChan))
		for i in range(nJD):
			if simDict['isMasked']:
				swfd[i,:] = unmaskCompressedData(n.abs( simDict['vis'][pol][b+nBL*i] ), simDict['msk'][pol][b+nBL*i])
			else:
				swfd[i,:] = n.abs( simDict['vis'][pol][b+nBL*i] )

	# Determine intelligent clipping levels
	hist, bins = n.histogram(wfd.compress(n.isfinite(wfd.ravel())), bins=1000)
	cdf = 1.0*hist.cumsum() / hist.sum()
	lowerClip = (n.where( cdf >= 0.05, bins[1:], n.zeros_like(bins[1:])+1e12)).min()
	upperClip = (n.where( cdf <= 0.90, bins[1:], n.zeros_like(bins[1:])-1e12)).max()
	print lowerClip, upperClip

	# Plot it all up
	ax1.imshow(wfd, origin='lower', vmin=lowerClip, vmax=upperClip, 
				extent=(dataDict['freq'][0]/1e6, dataDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))
	if simDict is not None:
		# Determine intelligent clipping levels
		hist, bins = n.histogram(swfd.compress(n.isfinite(swfd.ravel())), bins=1000)
		cdf = 1.0*hist.cumsum() / hist.sum()
		lowerClip = (n.where( cdf >= 0.05, bins[1:], n.zeros_like(bins[1:])+1e12)).min()
		upperClip = (n.where( cdf <= 0.95, bins[1:], n.zeros_like(bins[1:])-1e12)).max()
		print lowerClip, upperClip

		ax2.imshow(swfd, origin='lower', vmin=lowerClip, vmax=upperClip, 
					extent=(simDict['freq'][0]/1e6, simDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))

	if stands is None:
		title = 'Visibility Amplitudes for %i-%i' % dataDict['bls'][pol][baseline]
	else:
		i,j = dataDict['bls'][pol][baseline]
		title = 'Visibility Amplitudes for %i-%i' % (stands[i], stands[j])
	ax1.set_title(title)	
	ax1.set_xlabel('Frequency [MHz]')
	ax1.set_ylabel('Time [JD-%.4f]' % jds[0])
	ax1.axis('auto')

	if simDict is not None:
		if stands is None:
			title = 'Visibility Amplitudes for %i-%i (Sim.)' % simDict['bls'][pol][baseline]
		else:
			i,j = simDict['bls'][pol][baseline]
			title = 'Visibility Amplitudes for %i-%i (Sim.)' % (stands[i], stands[j])
		ax2.set_title(title)	
		ax2.set_xlabel('Frequency [MHz]')
		ax2.set_ylabel('Time [JD-%.4f]' % jds[0])
		ax2.axis('auto')

	plt.show()

	if SaveFig:
		fig.savefig('match-waterfall.png')


def plotDiffWaterfall(dataDict, simDict, baseline=21, pol='yy', stands=None, SaveFig=False):
	"""Create a waterfall diagram (visibility amplitude as a function of time and 
	frequency) for a particular baseline.  If simulated data are also provided, 
	plot those on a seperate axis."""
	
	fig = plt.figure()
	ax1 = fig.add_subplot(3, 1, 1)
	ax2 = fig.add_subplot(3, 1, 2)
	ax3 = fig.add_subplot(3, 1, 3)

	# Number of channels in the data set
	nChan = len(dataDict['freq'])

	# Build a list of unique Julian dates present in the data
	junk = {}
	for dt in dataDict['jd'][pol]:
		junk[dt] = 1
	jds = sorted(junk.keys())
	nJD = len(jds)
	del(junk)

	# Number of baselines in the dataset
	nBL = len(dataDict['vis'][pol]) / nJD
	baseline = baseline % nBL

	# Create the waterfall diagram for the data
	wfd = n.zeros((nJD,nChan))
	for i in range(nJD):
		if dataDict['isMasked']:
			wfd[i,:] = unmaskCompressedData(n.abs( dataDict['vis'][pol][baseline+nBL*i] ), dataDict['msk'][pol][baseline+nBL*i])
		else:
			wfd[i,:] = n.abs( dataDict['vis'][pol][baseline+nBL*i] )
	
	# Do the same for the simulated data if it is provided
	swfd = n.zeros((nJD,nChan))
	for i in range(nJD):
		if simDict['isMasked']:
			swfd[i,:] = unmaskCompressedData(n.abs( simDict['vis'][pol][baseline+nBL*i] ), simDict['msk'][pol][baseline+nBL*i])
		else:
			swfd[i,:] = n.abs( simDict['vis'][pol][baseline+nBL*i] )

	# Scale and difference
	diff = wfd - swfd*n.median(wfd)/n.median(swfd)
	print diff.min(), diff.mean(), diff.max()

	# Determine intelligent clipping levels
	hist, bins = n.histogram(wfd.compress(n.isfinite(wfd.ravel())), bins=1000)
	cdf = 1.0*hist.cumsum() / hist.sum()
	lowerClip = (n.where( cdf >= 0.05, bins[1:], n.zeros_like(bins[1:])+1e12)).min()
	upperClip = (n.where( cdf <= 0.95, bins[1:], n.zeros_like(bins[1:])-1e12)).max()
	print lowerClip, upperClip

	# Plot it all up
	ax1.imshow(wfd, origin='lower', vmin=lowerClip, vmax=upperClip, 
				extent=(dataDict['freq'][0]/1e6, dataDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))
	
	# Determine intelligent clipping levels
	hist, bins = n.histogram(swfd.compress(n.isfinite(swfd.ravel())), bins=1000)
	cdf = 1.0*hist.cumsum() / hist.sum()
	lowerClip = (n.where( cdf >= 0.05, bins[1:], n.zeros_like(bins[1:])+1e12)).min()
	upperClip = (n.where( cdf <= 0.95, bins[1:], n.zeros_like(bins[1:])-1e12)).max()
	print lowerClip, upperClip

	ax2.imshow(swfd, origin='lower', vmin=lowerClip, vmax=upperClip, 
				extent=(simDict['freq'][0]/1e6, simDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))

	ax3.imshow(diff, origin='lower', 
				extent=(simDict['freq'][0]/1e6, simDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))

	if stands is None:
		title = 'Visibility Amplitudes for %i-%i' % dataDict['bls'][pol][baseline]
	else:
		i,j = dataDict['bls'][pol][baseline]
		title = 'Visibility Amplitudes for %i-%i' % (stands[i], stands[j])
	ax1.set_title(title)	
	ax1.set_xlabel('Frequency [MHz]')
	ax1.set_ylabel('Time [JD-%.4f]' % jds[0])
	ax1.axis('auto')

	if stands is None:
		title = 'Visibility Amplitudes for %i-%i (Sim.)' % simDict['bls'][pol][baseline]
	else:
		i,j = simDict['bls'][pol][baseline]
		title = 'Visibility Amplitudes for %i-%i (Sim.)' % (stands[i], stands[j])
	ax2.set_title(title)	
	ax2.set_xlabel('Frequency [MHz]')
	ax2.set_ylabel('Time [JD-%.4f]' % jds[0])
	ax2.axis('auto')

	if stands is None:
		title = 'Visibility Amplitudes for %i-%i (Diff.)' % simDict['bls'][pol][baseline]
	else:
		i,j = simDict['bls'][pol][baseline]
		title = 'Visibility Amplitudes for %i-%i (Diff.)' % (stands[i], stands[j])
	ax3.set_title(title)	
	ax3.set_xlabel('Frequency [MHz]')
	ax3.set_ylabel('Time [JD-%.4f]' % jds[0])
	ax3.axis('auto')

	plt.show()

	if SaveFig:
		fig.savefig('diff-waterfall.png')


def plotPhaseWaterfall(dataDict, baseline=21, pol='yy', stands=None, simDict=None, SaveFig=False):
	"""Create a waterfall diagram (visibility phase as a function of time and 
	frequency) for a particular baseline.  If simulated data are also provided, 
	plot those on a seperate axis."""
	
	fig = plt.figure()
	if simDict is None:
		ax1 = fig.add_subplot(1, 1, 1)
	else:
		ax1 = fig.add_subplot(2, 1, 1)
		ax2 = fig.add_subplot(2, 1, 2)

	# Number of channels in the data set
	nChan = len(dataDict['freq'])

	# Build a list of unique Julian dates present in the data
	junk = {}
	for dt in dataDict['jd'][pol]:
		junk[dt] = 1
	jds = sorted(junk.keys())
	nJD = len(jds)
	del(junk)

	# Number of baselines in the dataset
	nBL = len(dataDict['vis'][pol]) / nJD
	baseline = baseline % nBL

	# Create the waterfall diagram for the data
	wfd = n.zeros((nJD,nChan))
	for i in range(nJD):
		phs = argument(dataDict['vis'][pol][baseline+nBL*i])
		if dataDict['isMasked']:
			wfd[i,:] = unmaskCompressedData(phs, dataDict['msk'][pol][baseline+nBL*i])
		else:
			wfd[i,:] = phs
	
	# Do the same for the simulated data if it is provided
	if simDict is not None:
		swfd = n.zeros((nJD,nChan))
		for i in range(nJD):
			phs = argument(simDict['vis'][pol][baseline+nBL*i])
			if simDict['isMasked']:
				swfd[i,:] = unmaskCompressedData(phs, simDict['msk'][pol][baseline+nBL*i])
			else:
				swfd[i,:] = phs

	# Plot it all up
	ax1.imshow(wfd, origin='lower', vmin=-n.pi, vmax=n.pi, 
				extent=(dataDict['freq'][0]/1e6, dataDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))
	if simDict is not None:
		ax2.imshow(swfd, origin='lower', vmin=-n.pi, vmax=n.pi, 
					extent=(simDict['freq'][0]/1e6, simDict['freq'][-1]/1e6, 0, jds[-1]-jds[0]))

	if stands is None:
		title = 'Visibility Phases for %i-%i' % dataDict['bls'][pol][baseline]
	else:
		i,j = dataDict['bls'][pol][baseline]
		title = 'Visibility Phases for %i-%i' % (stands[i], stands[j])
	ax1.set_title(title)	
	ax1.set_xlabel('Frequency [MHz]')
	ax1.set_ylabel('Time [JD-%.4f]' % jds[0])
	ax1.axis('auto')

	if simDict is not None:
		if stands is None:
			title = 'Visibility Phases for %i-%i (Sim.)' % simDict['bls'][pol][baseline]
		else:
			i,j = simDict['bls'][pol][baseline]
			title = 'Visibility Phases for %i-%i (Sim.)' % (stands[i], stands[j])
		ax2.set_title(title)	
		ax2.set_xlabel('Frequency [MHz]')
		ax2.set_ylabel('Time [JD-%.4f]' % jds[0])
		ax2.axis('auto')

	plt.show()

	if SaveFig:
		fig.savefig('phase-waterfall.png')


# -*- coding: utf-8 -*-

"""Unit test for the lsl.writer.tsfits modules."""

import os
import time
import unittest
import tempfile
import numpy
import pyfits

from lsl.common.paths import dataBuild as dataPath
from lsl.reader import tbw
from lsl.reader import tbn
from lsl.reader import errors
from lsl.writer import vdif


__revision__ = "$Revision:1 $"
__version__  = "0.1"
__author__    = "Jayce Dowell"

tbwFile = os.path.join(dataPath, 'tests', 'tbw-test.dat')
tbnFile = os.path.join(dataPath, 'tests', 'tbn-test.dat')


class vdif_tests(unittest.TestCase):
	"""A unittest.TestCase collection of unit tests for the lsl.writer.vdif
	module."""

	def __getTBW(self):
		"""Private function to load in the test TBW data and get the frames."""

		fh = open(tbwFile, 'rb')

		# Frames 1 through 8
		frames = []
		for i in range(1,9):
			frames.append(tbw.readFrame(fh))

		return frames

	def __getTBN(self, vanilla=False):
		"""Private function to load in the test TBN data and get the frames.  If 
		the keyword 'vanilla' is set to True, gain, sample rate, and frequency meta-
		data are not added to the frames."""

		fh = open(tbnFile, 'rb')

		# Frames 1 through 8
		frames = []
		for i in range(1,9):
			frames.append(tbn.readFrame(fh))

		if not vanilla:
			# Set some values for the other meta-data
			for frame in frames:
				frame.setCentralFreq(40e6)
				frame.setGain(22)
				frame.setSampleRate(100000)

		return frames

	def test_vdif_real(self):
		"""Test writing real data to VDIF format."""

		# Setup the file names
		testPath = tempfile.mkdtemp(prefix='tsfits-', suffix='.tmp')
		testFile = os.path.join(testPath, 'ts-tbw-testS.fits')

		# Get some data
		frames = self.__getTBW()

		# Write the data
		fh = open(testFile, 'wb')
		for frame in frames:
			vFrame = vdif.Frame(frame.parseID(), frame.getTime(), bits=12, data=frame.data.xy[0,:])
			vFrame.writeRawFrame(fh)
		fh.close()

		# Read it back in
		fh = open(testFile, 'rb')
		junk = numpy.fromfile(fh, dtype=numpy.uint8, count=16)
		frameSize = (junk[10]<<16) | (junk[9]<<8) | (junk[8])
		dataBits = (junk[15]>>2) & 15
		dataSize = (frameSize - 4) * 8
		fh.seek(0)
		vFrames = []
		for i in range(8):
			vFrames.append( numpy.fromfile(fh, dtype=numpy.uint8, count=frameSize*8) )
		for vFrame, tFrame in zip(vFrames, frames):
			data = numpy.zeros(400, dtype=numpy.int16)
			j = 0
			for i in range(32,vFrame.shape[0],4):
				word = vFrame[i] | (vFrame[i+1]<<8) | (vFrame[i+2]<<16) | (vFrame[i+3]<<24)
				for k in range(32/dataBits):
					data[j] = ((word>>(dataBits*k)) & (2**dataBits-1)) - 2**(dataBits-1)
					j = j + 1
			
			for v,t in zip(data, tFrame.data.xy[0,:]):
				self.assertEqual(v, t)

		fh.close()
		os.unlink(testFile)
		os.rmdir(testPath)

	def test_vdif_complex(self):
		"""Test writing complex data to VIDF format."""

		# Setup the file names
		testPath = tempfile.mkdtemp(prefix='tsfits-', suffix='.tmp')
		testFile = os.path.join(testPath, 'ts-tbw-testS.fits')

		# Get some data
		frames = self.__getTBN()
		
		# Write the data
		fh = open(testFile, 'wb')
		for frame in frames:
			stand, pol = frame.parseID()
			if pol == 1:
				continue
			vFrame = vdif.Frame(stand, frame.getTime(), bits=8, data=frame.data.iq)
			vFrame.writeRawFrame(fh)
		fh.close()

		# Read it back in
		fh = open(testFile, 'rb')
		junk = numpy.fromfile(fh, dtype=numpy.uint8, count=16)
		frameSize = (junk[10]<<16) | (junk[9]<<8) | (junk[8])
		dataBits = (junk[15]>>2) & 15
		dataSize = (frameSize - 4) * 8
		fh.seek(0)
		vFrames = []
		for i in range(8):
			vFrames.append( numpy.fromfile(fh, dtype=numpy.uint8, count=frameSize*8) )
		for vFrame, tFrame in zip(vFrames, frames[::2]):
			data = numpy.zeros(512, dtype=numpy.singlecomplex)
			j = 0
			for i in range(32,vFrame.shape[0],4):
				word = vFrame[i] | (vFrame[i+1]<<8) | (vFrame[i+2]<<16) | (vFrame[i+3]<<24)
				for k in range(32/dataBits):
					if k % 2 == 0:
						data.real[j] = ((word>>(dataBits*k)) & (2**dataBits-1)) - 2**(dataBits-1)
					else:
						data.imag[j] = ((word>>(dataBits*k)) & (2**dataBits-1)) - 2**(dataBits-1)
						j = j + 1

			for v,t in zip(data, tFrame.data.iq):
				self.assertAlmostEqual(v, t, 6)
		
		fh.close()
		os.unlink(testFile)
		os.rmdir(testPath)


class  vdif_test_suite(unittest.TestSuite):
	"""A unittest.TestSuite class which contains all of the lsl.writer.vdif units 
	tests."""
	
	def __init__(self):
		unittest.TestSuite.__init__(self)
		
		loader = unittest.TestLoader()
		self.addTests(loader.loadTestsFromTestCase(vdif_tests)) 


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

"""Python module for reading data in from TBN files.This module defines the 
following classes for storing the TBN data found in a file:

Frame
  object that contains all data associated with a particular TBN frame.  
  The primary constituents of each frame are:
    * FrameHeader - the TBN frame header object and
    * FrameData   - the TBN frame data object.
  Combined, these two objects contain all of the information found in the 
  original TBN frame.

ObservingBlock
  object that stores a collection of Frames for all stands/polarizations for 
  a particular time.

In addition to storing the data available in the frame, the Frame object also
has attributes for holding information about the gain, central frequency, and
filter code used for the observations.

The functions defined in this module fall into two class:
 1. convert a frame in a file to a Frame object and
 2. describe the format of the data in the file.

For reading in data, use the readFrame function.  It takes a python file-
handle as an input and returns a fully-filled Frame object.  The readBlock
function reads in a (user-defined) number of TBN frames and returns a 
ObservingBlock object.

For describing the format of data in the file, two function are provided:

getSampleRate
  read in the few frame of an open file handle and return the sampling rate 
  of the data

getFramesPerObs
  read in the first several frames to see how many stands are found in the data.
"""

import os
import sys
import time
import numpy
import array
import struct

from  lsl.common import dp as dp_common
from errors import *

__version__ = '0.5'
__revision__ = '$ Revision: 10 $'
__all__ = ['FrameHeader', 'FrameData', 'Frame', 'ObservingBlock', 'readFrame', 'readBlock', 'getSampleRate', 'getFramesPerObs', 'FrameSize', 'filterCodes', '__version__', '__revision__', '__all__']

FrameSize = 1048

# List of filter codes and their corresponding sample rates in Hz
filterCodes = {1: 1000, 2: 3125, 3: 6250, 4: 12500, 5: 25000, 6: 50000, 7: 100000}


class FrameHeader(object):
	"""Class that stores the information found in the header of a TBW 
	frame.  All three fields listed in the DP IDC version H are stored as 
	well as the original binary header data."""

	def __init__(self, frameCount=None, secondsCount=None, tbnID=None,  raw=None):
		self.frameCount = frameCount
		self.secondsCount = secondsCount
		self.tbnID = tbnID
		self.raw = raw
		
	def isTBN(self):
		"""Function to check if the data is really TBN and not TBW by examining
		the TBN ID field.  Returns True if the data is TBN, false otherwise."""

		mode = (self.tbnID>>15)&1
		if mode == 0:
			return True
		else:
			return False

	def parseID(self):
		"""Function to parse the TBN ID field and return a tuple of the stand 
		number and polarization."""

		if self.tbnID&1023 % 2 == 0:
			stand = (self.tbnID&1023) / 2
			pol = 1
		else:
			stand = (self.tbnID&1023) / 2 + 1
			pol = 0
			
		return (stand, pol)


class FrameData(object):
	"""Class that stores the information found in the data section of a TBN
	frame.  Both fields listed in the DP IDC version H are stored."""

	def __init__(self, timeTag=None, iq=None):
		self.sampleRate = None
		self.centralFreq = None
		self.gain = None
		self.timeTag = timeTag
		self.iq = iq

	def getTime(self):
		"""Function to convert the time tag from samples since station midnight to
		seconds since station midnight.  This function needs to dp_common module 
		in order to work."""
		
		return self.timeTag / dp_common.fS

	def getFilterCode(self):
		"""Function to convert the sample rate in Hz to a filter code."""
		if self.sampleRate is None:
			return None
		else:
			sampleCodes = {}
			for key,value in filterCodes.iteritems():
				sampleCodes[value] = key

			return sampleCodes[self.sampleRate]

	def setSampleRate(self, sampleRate):
		"""Function to set the sample rate of the TBN data in Hz."""

		self.sampleRate = sampleRate

	def setCentralFreq(self, centralFreq):
		"""Function to set the central frequency of the TBN data in Hz."""

		self.centralFreq = centralFreq

	def setGain(self, gain):
		"""Function to set the gain of the TBN data."""

		self.gain = gain


class Frame(object):
	"""Class that stores the information contained within a single TBN 
	frame.  It's properties are FrameHeader and FrameData objects."""

	def __init__(self, header=FrameHeader(), data=FrameData()):
		self.header = header
		self.data = data
		self.valid = True

	def parseID(self):
		"""Convenience wrapper for the Frame.FrameHeader.parseID function."""
		
		return self.header.parseID()

	def getTime(self):
		"""Convenience wrapper for the Frame.FrameData.getTime function."""
		
		return self.data.getTime()

	def getFilterCode(self):
		"""Convenience wrapper for the Frame.FrameData.getFilterCode function."""

		return self.data.getFilterCode()

	def setSampleRate(self, sampleRate):
		"""Convenience wrapper for the Frame.FrameData.setSampleRate function."""

		self.data.setSampleRate(sampleRate)

	def setCentralFreq(self, centralFreq):
		"""Convenience wrapper for the Frame.FrameData.setCentralFreq function."""

		self.data.setCentralFreq(centralFreq)

	def setGain(self, gain):
		"""Convenience wrapper for the Frame.FrameData.setGain function."""

		self.data.setGain(gain)
			
	def __add__(self, y):
		"""Add the data sections of two frames together or add a number 
		to every element in the data section."""
		
		try:
			return self.data.xy + y.self.data.xy
		except AttributeError:
			return self.data.xy + y
			
	def __iadd__(self, y):
		"""In-place add the data sections of two frames together or add 
		a number to every element in the data section."""
		
		try:
			self.data.xy += y.self.data.xy
		except AttributeError:
			self.data.xy += y
		
	def __mul__(self, y):
		"""Multiple the data sections of two frames together or multiply 
		a number to every element in the data section."""
		
		try:
			return self.data.xy * y.self.data.xy
		except AttributeError:
			return self.data.xy * y
			
	def __imul__(self, y):
		"""In-place multiple the data sections of two frames together or 
		multiply a number to every element in the data section."""
		
		try:
			self.data.xy *= y.self.data.xy
		except AttributeError:
			self.data.xy *= y

	def __eq__(self, y):
		"""Check if the time tags of two frames are equal or if the time
		tag is equal to a particular value."""
		
		tX = self.data.timeTag
		try:
			tY = self.data.timeTag
		except AttributeError:
			tY = y
		
		if tX == tY:
			return True
		else:
			return False
			
	def __ne__(self, y):
		"""Check if the time tags of two frames are not equal or if the time
		tag is not equal to a particular value."""
		
		tX = self.data.timeTag
		try:
			tY = self.data.timeTag
		except AttributeError:
			tY = y
		
		if tX != tY:
			return True
		else:
			return False
			
	def __gt__(self, y):
		"""Check if the time tag of the first frame is greater than that of a
		second frame or if the time tag is greater than a particular value."""
		
		tX = self.data.timeTag
		try:
			tY = self.data.timeTag
		except AttributeError:
			tY = y
		
		if tX > tY:
			return True
		else:
			return False
			
	def __ge__(self, y):
		"""Check if the time tag of the first frame is greater than or equal to 
		that of a second frame or if the time tag is greater than a particular 
		value."""
		
		tX = self.data.timeTag
		try:
			tY = self.data.timeTag
		except AttributeError:
			tY = y
		
		if tX >= tY:
			return True
		else:
			return False
			
	def __lt__(self, y):
		"""Check if the time tag of the first frame is less than that of a
		second frame or if the time tag is greater than a particular value."""
		
		tX = self.data.timeTag
		try:
			tY = self.data.timeTag
		except AttributeError:
			tY = y
		
		if tX < tY:
			return True
		else:
			return False
			
	def __le__(self, y):
		"""Check if the time tag of the first frame is less than or equal to 
		that of a second frame or if the time tag is greater than a particular 
		value."""
		
		tX = self.data.timeTag
		try:
			tY = self.data.timeTag
		except AttributeError:
			tY = y
		
		if tX <= tY:
			return True
		else:
			return False
			
	def __cmp__(self, y):
		"""Compare two frames based on the time tags.  This is helpful for 
		sorting things."""
		
		tX = self.data.timeTag
		tY = y.data.timeTag
		if tY > tX:
			return -1
		elif tX > tY:
			return 1
		else:
			return 0


class ObservingBlock(object):
	def __init__(self, x=[], y=[]):
		self.x = x
		self.y = y

	def getTime(self):
		"""Convenience wrapper for the Frame.FrameData.getTime function."""
		
		return self.x[0].data.getTime()

	def getFilterCode(self):
		"""Convenience wrapper for the Frame.FrameData.getFilterCode function."""

		return self.x[0].data.getFilterCode()

	def setSampleRate(self, sampleRate):
		"""Convenience wrapper for the Frame.FrameData.setSampleRate function."""

		for i in len(self.x):
			self.x[i].data.setSampleRate(sampleRate)
		for i in len(self.y):
			self.y[i].data.setSampleRate(sampleRate)

	def setCentralFreq(self, centralFreq):
		"""Convenience wrapper for the Frame.FrameData.setCentralFreq function."""

		for i in len(self.x):
			self.x[i].data.setCentralFreq(centralFreq)
		for i in len(self.y):
			self.y[i].data.setCentralFreq(centralFreq)

	def setGain(self, gain):
		"""Convenience wrapper for the Frame.FrameData.setGain function."""

		for i in len(self.x):
			self.x[i].data.setGain(gain)
		for i in len(self.y):
			self.y[i].data.setGain(gain)


def __readHeader(filehandle, Verbose=False):
	"""Private function to read in a TBN header.  Returns a FrameHeader object."""

	rawHeader = ''
	try:
		s = filehandle.read(4)
		rawHeader = rawHeader + s
		sync4, sync3, sync2, sync1 = struct.unpack(">BBBB", s)
		s = filehandle.read(4)
		rawHeader = rawHeader + s
		m5cID, frameCount3, frameCount2, frameCount1 = struct.unpack(">BBBB", s)
		frameCount = (long(frameCount3)<<16) | (long(frameCount2)<<8) | long(frameCount1)
		s = filehandle.read(4)
		rawHeader = rawHeader + s
		secondsCount = struct.unpack(">L", s)
		s = filehandle.read(4)
		rawHeader = rawHeader + s
		tbnID, junk = struct.unpack(">HH", s)
	except IOError:
		raise eofError()
	except struct.error:
		raise eofError()

	if sync1 != 92 or sync2 != 222 or sync3 != 192 or sync4 != 222:
		raise syncError(sync1=sync1, sync2=sync2, sync3=sync3, sync4=sync4)

	newHeader = FrameHeader()
	newHeader.frameCount = frameCount
	newHeader.secondsCount = secondsCount[0]
	newHeader.tbnID = tbnID
	newHeader.raw = rawHeader

	if Verbose:
		stand, pol = newHeader.parseID()
		print "Header: ", tbnID
		print "  Stand: ", stand
		print "  Polarization: ", pol

	return newHeader


def __readData(filehandle):
	"""Private function to read in a TBN frame data section.  Returns a 
	FrameData object."""

	try:
		s = filehandle.read(8)
		timeTag = struct.unpack(">Q", s)
	except IOError:
		raise eofError()

	rawData = numpy.fromfile(filehandle, dtype=numpy.uint8, count=1024)
	rawData = rawData.astype(numpy.int8)
	data = numpy.zeros(512, dtype=numpy.csingle)
	if rawData.shape[0] < 2*data.shape[0]:
		raise numpyError()

	# The data are signed, so apply the two-complement rule to generate 
	# the negative values
	negativeValues = numpy.where( rawData >= 128 )
	rawData[negativeValues] -= 256

	data.real = rawData[0::2]
	data.imag = rawData[1::2]
	#print data.real.min(), data.real.max(), data.imag.min(), data.imag.max()
	
	newData = FrameData()
	newData.timeTag = timeTag[0]
	newData.iq = data

	return newData


def readFrame(filehandle, SampleRate=None, CentralFreq=None, Gain=None, Verbose=False):
	"""Function to read in a single TBN frame (header+data) and store the 
	contents as a Frame object.  This function wraps readerHeader and 
	readData."""

	try:
		hdr = __readHeader(filehandle, Verbose=Verbose)
	except syncError, err:
		# Why?  If we run into a sync error here, then the following frame is invalid.  
		# Thus, we need to skip over this frame be advancing the file pointer 8+1000 B 
		currPos = filehandle.tell()
		frameEnd = currPos + FrameSize - 16
		filehandle.seek(frameEnd)
		raise err

	dat = __readData(filehandle)
	
	newFrame = Frame()
	newFrame.header = hdr
	newFrame.data = dat
	
	newFrame.setSampleRate(SampleRate)
	newFrame.setCentralFreq(CentralFreq)
	newFrame.setGain(Gain)

	return newFrame


def readBlock(filehandle, nFrames=512, SampleRate=None, CentralFreq=None, Gain=None, Verbose=False):
	"""Function to read in a single TBW block (frames set by the nFrames 
	keyword) and store the contents as a ObservingBlock object.  This function 
	wraps readFrame."""

	frames = []
	for i in range(0, nFrames):
		try:
			frame = readFrame(filehandle, SampleRate=SampleRate, CentralFreq=CentralFreq, Gain=Gain, Verbose=Verbose)
		except baseReaderError:
			frame = None
		frames.append( frame )

	return ObservingBlock(x=frames[0::2], y=frames[1::2])
	

def getSampleRate(filehandle, nFrames=None, FilterCode=False):
	"""Find out what the sampling rate/filter code is from consecutive sets of 
	observations.  By default, the rate in Hz is returned.  However, the 
	corresponding filter code can be returned instead by setting the FilterCode
	keyword to true."""

	# Save the current position in the file so we can return to that point
	fhStart = filehandle.tell()
	
	# Go back to the beginning...
	filehandle.seek(0)

	if nFrames is None:
		nFrames = 512
	nFrames = 2*nFrames

	# Build up the list-of-lists that store ID codes and loop through 512
	# frames.  In each case, parse pull the TBN ID, extract the stand 
	# number, and append the stand number to the relevant polarization array 
	# if it is not already there.
	frames = {}
	for i in range(nFrames):
		try:
			cFrame = readFrame(filehandle)
		except eofError:
			break
		except syncError:
			continue
		except numpyError:
			break
		
		stand, pol = cFrame.parseID()
		key = 2*stand + pol
		if key not in frames.keys():
			frames[key] = []
		frames[key].append(cFrame)
			
	# Return to the place in the file where we started
	filehandle.seek(fhStart)

	# Any key with complete data will work for this, so pick the first key with two
	# valid frames
	keyCount = 0
	frame1 = None
	frame2 = None
	while frame1 is None and frame2 is None:
		validKey = (frames.keys())[keyCount]
		frame1 = frames[validKey][0]
		frame2 = frames[validKey][1]

		keyCount = keyCount + 1

	# Now that we have two valid frames that follow one another in time, load in their
	# time tags and calculate the sampling rate.  Since the time tags are based off f_S
	# @ 196 MSPS, and each frame contains 512 samples, the sampleing rate is:
	#  f_S / <difference in time tags per 512 samples>
	time1 = frame1.data.timeTag
	time2 = frame2.data.timeTag
	rate = dp_common.fS / (abs( time2 - time1 ) / 512)

	if not FilterCode:
		return rate
	else:
		sampleCodes = {}
		for key,value in filterCodes.iteritems():
			sampleCodes[value] = key

		return sampleCodes[rate]


def getFramesPerObs(filehandle):
	"""Find out how many frames are present per observation by examining 
	the first 512 TBN frames.  Return the number of frames per observations 
	as a two-	element tuple, one for each polarization."""
	
	# Save the current position in the file so we can return to that point
	fhStart = filehandle.tell()
	
	# Go back to the beginning...
	filehandle.seek(0)

	# Build up the list-of-lists that store ID codes and loop through 512
	# frames.  In each case, parse pull the TBN ID, extract the stand 
	# number, and append the stand number to the relevant polarization array 
	# if it is not already there.
	idCodes = [[], []]
	for i in range(512):
		try:
			cFrame = readFrame(filehandle)
		except eofError:
			break
		except syncError:
			continue
		except numpyError:
			break
		
		cID, cPol = cFrame.header.parseID()
		if cID not in idCodes[cPol]:
			idCodes[cPol].append(cID)
			
	# Return to the place in the file where we started
	filehandle.seek(fhStart)
	
	# Get the length of each beam list and return them as a tuple
	return (len(idCodes[0]), len(idCodes[1]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Example script for reading in TBW data and writing it to a TS-FITS file.
This version differs from the regular readTBN script in that it uses a frame
buffer to reorder out-of-order packets and dropped frames."""

import os
import sys
import time
from lsl.reader import tbn
from lsl.reader import errors
from lsl.reader.buffer import TBNFrameBuffer as FrameBuffer
from lsl.writer import tsfits

import matplotlib.pyplot as plt

def main(args):
	nSamples = os.path.getsize(args[0]) / tbn.FrameSize
	print "Samples in file: ", nSamples
	fh = open(args[0], "rb", buffering=tbn.FrameSize)
	
	test = tbn.readFrame(fh)
	print "TBN Data:  %s" % test.header.isTBN()
	if not test.header.isTBN():
		raise notTBNError()
	fh.seek(0)

	nFpO = tbn.getFramesPerObs(fh)
	print "Samples per observations: %i in x pol., %i in y pol." % nFpO
	nFpO = nFpO[0] + nFpO[1]

	sampleRate = tbn.getSampleRate(fh, nFrames=nFpO)
	print "Filter code is: %i" % tbn.getSampleRate(fh, nFrames=nFpO, FilterCode=True)
	print "Sampling rate is: %i Hz" % sampleRate

	tStart = time.time()

	# Create the FrameBuffer instance
	buffer = FrameBuffer(stands=range(nFpO/2), pols=[0, 1])

	# Create a new FITS file with the name 'tbw.fits'
	fitsFile = tsfits.TBN('tbn-tsfits-test.fits')

	nSamples = 340000

	blocks = []
	count = {}
	syncCount = 0
	masterCount = 0
	for i in range(nSamples):
		try:
			cFrame = tbn.readFrame(fh, SampleRate=sampleRate)
		except errors.eofError:
			break
		except errors.syncError:
			syncCount = syncCount + 1
			continue
		except errors.numpyError:
			break
		buffer.append(cFrame)
		cFrames = buffer.get()
		if cFrames is None:
			continue
		blocks.append(cFrames)

		for frame in blocks[-1]:
			if frame is None:
				print "sync error"
				continue
			stand, pol = frame.parseID()
			if stand not in count.keys():
				count[stand] = 0
			count[stand] = count[stand] + 1

		for cFrame in blocks[-1]:
			fitsFile.addStandData(cFrame)

		masterCount = masterCount + 1
		#print cFrame.header.parseID(),cFrame.data.timeTag

	tEnd = time.time()
	print 'Read %i frames in %0.3f s (%0.1f frames/s)' % (nFpO*nSamples, (tEnd-tStart), nFpO*nSamples/(tEnd-tStart))

	fh.close()
	fitsFile.close()
	fitsFile.info()
	buffer.status()

	# Summary information about the file that was just read in
	print "Summary:"
	for stand in sorted(count.keys()):
		print "Stand: %2i, Frames: %5i" % (stand, count[stand])
	print "Sync Errors: %5i" % syncCount


if __name__ == "__main__":
	main(sys.argv[1:])



# -*- coding: utf-8 -*-

"""Unit test for the lsl.sim.vis module."""

import os
import unittest
import numpy

from lsl.sim import vis
from lsl.common import stations as lwa_common


__revision__ = "$Revision:1 $"
__version__  = "0.1"
__author__    = "Jayce Dowell"


class simvis_tests(unittest.TestCase):
	"""A unittest.TestCase collection of unit tests for the lsl.sim.vis
	module."""

	def setUp(self):
		"""Turn off all numpy warnings."""

		numpy.seterr(all='ignore')

	def test_build_aa_flat(self):
		"""Test building a antenna array object with uniform sky response."""

		lwa1 = lwa_common.lwa1()
		stands = lwa1.getStands()
		freqs = numpy.arange(30e6, 50e6, 1e6)

		aa = vis.buildSimArray(lwa1, stands, freqs, ForceFlat=True)

	def test_build_aa(self):
		"""Test building a antenna array object with realistic sky response."""

		lwa1 = lwa_common.lwa1()
		stands = lwa1.getStands()
		freqs = numpy.arange(30e6, 50e6, 1e6)

		aa = vis.buildSimArray(lwa1, stands, freqs)
		# Check the number of stands
		self.assertEqual(len(aa.ants), len(stands))

		# Check the frequencies comming out
		for fo, fi in zip(aa.get_afreqs(), freqs):
			self.assertAlmostEqual(fo, fi/1e9, 6)

	def test_build_data(self):
		"""Test building simulated visibility data"""

		# Setup
		lwa1 = lwa_common.lwa1()
		stands = lwa1.getStands()
		freqs = numpy.arange(30e6, 50e6, 1e6)
		aa = vis.buildSimArray(lwa1, stands, freqs)

		# Build the data dictionary
		out = vis.buildSimData(aa, vis.srcs)

		# Do a check of keys
		keyList = out.keys()
		for key in ['freq', 'isMasked', 'bls', 'uvw', 'vis', 'wgt', 'msk', 'jd']:
			self.assertTrue(key in keyList)

		# Do a check of frequencies
		for fa, fq in zip(out['freq'], freqs):
			self.assertAlmostEqual(fa, fq, 6)

		# Do a check to make sure that the entries with secondary keys have them
		for key in ['bls', 'uvw', 'vis', 'wgt', 'msk', 'jd']:
			secondaryKeyList = out[key].keys()
			for key2 in ['xx', 'yy', 'xy', 'yx']:
				self.assertTrue(key2 in secondaryKeyList)

		# Do a check to make sure that the entries with secondary keys also 
		# have lists in them
		for key in ['bls', 'uvw', 'vis', 'wgt', 'msk', 'jd']:
			secondaryKeyList = out[key].keys()
			for key2 in ['xx', 'yy', 'xy', 'yx']:
				self.assertTrue(type(out[key][key2]).__name__ == 'list')
		
	def test_scale_data(self):
		"""Test that we can scale a data dictionary without error"""

		# Setup
		lwa1 = lwa_common.lwa1()
		stands = lwa1.getStands()
		freqs = numpy.arange(30e6, 50e6, 1e6)
		aa = vis.buildSimArray(lwa1, stands, freqs)

		# Build the data dictionary
		out = vis.buildSimData(aa, vis.srcs)

		# Scale
		amp = vis.scaleData(out, numpy.ones(len(stands))*2, numpy.zeros(len(stands)))
		# Delay
		phs = vis.scaleData(out, numpy.ones(len(stands)), numpy.ones(len(stands)))

	def test_shift_data(self):
		"""Test that we can shift the uvw coordinates of a data dictionary 
		without error"""

		# Setup
		lwa1 = lwa_common.lwa1()
		stands = lwa1.getStands()
		freqs = numpy.arange(30e6, 50e6, 1e6)
		aa = vis.buildSimArray(lwa1, stands, freqs)

		# Build the data dictionary
		out = vis.buildSimData(aa, vis.srcs)

		# Shift
		sft = vis.shiftData(out, aa)


class  simvis_test_suite(unittest.TestSuite):
	"""A unittest.TestSuite class which contains all of the lsl.sim.vis units 
	tests."""
	
	def __init__(self):
		unittest.TestSuite.__init__(self)

		loader = unittest.TestLoader()
		self.addTests(loader.loadTestsFromTestCase(simvis_tests)) 


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

"""Module that contains the error classes for the DRX, TBN, and TBW readers.  
These errors are currently meant to deal with file I/O problems."""

__version__ = '0.1'
__revision__ = '$ Revision: 0 $'
__all__ = ['baseReaderError', 'eofError', 'numpyError', 'syncError', 'notTBNError', 'notTBWError', 'listErrorCodes', 'MinErrorNo', 'MaxErrorNo', '__version__', '__revision__', '__all__']


MinErrorNo = 1
MaxErrorNo = 5


class baseReaderError(IOError):
	"""Base class for file I/O problem during numpy.fromfile calls and out-of-
sync Mark5C headers."""

	def __init__(self, strerror, errno='-1'):
		self.errno = errno
		self.strerror = strerror
		self.filename = None
		self.args = (errno, strerror)

	def __str__(self):
		return "%s" % self.strerror


class eofError(baseReaderError):
	"""Extension to the base class for dealing with EOF errors.  The error code
	is 1."""

	def __init__(self):
		self.errno = 1
		self.strerror = 'End of file encountered during filehandle read'
		self.filename = None
		self.args = (self.errno, self.strerror)


class numpyError(baseReaderError):
	"""Extension to the base class for dealing with errors that occur when 
	numpy.fromfile tried to read more data can exists.  This is a specialized form
	of EOF error.  The error code is 2."""

	def __init__(self):
		self.errno = 2
		self.strerror = 'End of file encountered during numpy.fromfile call'
		self.filename = None
		self.args = (self.errno, self.strerror)


class syncError(baseReaderError):
	"""Extension to the base class for dealing with Mark 5C header sync word 
	problems.  If the sync word doesn't match what is expected.  The error code 
	is 3."""

	def __init__(self, sync1=None, sync2=None, sync3=None, sync4=None):
		self.errno = 3
		self.strerror = 'Mark 5C sync word differs from expected'
		self.filename = None
		self.args = (self.errno, self.strerror)
		self.syncWord = (sync1, sync2, sync3, sync4)

	def __str__(self):
		if self.syncWord[0] is None:
			return self.strerror
		else:
			return "%s:  %2X %2X %2X %2X" % (self.strerror, self.syncWord[0], self.syncWord[1], self.syncWord[2], self.syncWord[3])


class notTBNError(baseReaderError):
	"""Extenstion to the base class for dealing with trying to read in TBW data 
	with a TBN reader.  The error code is 4."""

	def __init__(self):
		self.errno = 4
		self.strerror = 'Data appears to be TBW, not TBN as expected'
		self.filename = None
		self.args = (self.errno, self.strerror)


class notTBWError(baseReaderError):
	"""Extenstion to the base class for dealing with trying to read in TBN data 
	with a TBW reader.  The error code is 5."""

	def __init__(self):
		self.errno = 5
		self.strerror = 'Data appears to be TBN, not TBW as expected'
		self.filename = None
		self.args = (self.errno, self.strerror)


def listErrorCodes(errno=None):
	"""Function to provide a list of errors defined in this file.  It 
	alternatively takes an error code using the 'errno' keyword and returns its
	description."""

	if errno is None:
		for i in range(MinErrorNo, (MaxErrorNo+1)):
			listErrorCodes(errno=i)
	else:
		if errno == 1:
			print "1: End of file encountered during filehandle read"
		elif errno == 2:
			print "2: End of file encountered during numpy.fromfile call"
		elif errno == 3:
			print "3: Mark 5C sync word differs from expected"
		elif errno == 4:
			print "4: Data appears to be TBW, not TBN as expected"
		elif errno == 5:
			print "5: Data appears to be TBN, not TBW as expected"
		else:
			print "Unknown error code '%i'" % errno



# -*- coding: utf-8 -*-

"""Useful math functions for LWA work"""

import logging
import math

import numpy

__version__   = "0.1"
__revision__  = "$Revision: 92 $"
__all__ = ['regrid', 'downsample', 'smooth', 'cmagnitude', 'cphase', 'cpolar', 'crect', 'creal', 'cimag', 'to_dB', 'from_dB', 'robustmean', '__version__', '__revision__', '__all__']
__author__    = "P.S.Ray"
__maintainer__ = "Jayce Dowell"

_MATHUTIL_LOG = logging.getLogger('mathutil')


def regrid(x, y, newx, allow_extrapolation = False, method = 'spline'):
	"""Regrid data from x,y onto newx. If allow_extrapolation is True,
	extrapolation is attempted if the method supports it.  Supported
	methods are:
	  * linear
	  * spline
	Use of this function may require the scipy extension package."""
	
	if method == 'linear':
		return _regrid_linear(x, y, newx, allow_extrapolation)
	elif method == 'spline':
		return _regrid_spline(x, y, newx, allow_extrapolation)
	else:
		raise ValueError("interp method '%s' not recognized" % method)


def _regrid_linear(x, y, newx, allow_extrapolation=False):
	"""Implement regrid() function using linear interpolation."""
	
	if allow_extrapolation:
		_MATHUTIL_LOG.warning("allow_extrapolation=True not honored for regrid_linear")
	
	if newx.min() < x.min():
		raise ValueError('x.min(%f) must be smaller than newx.min(%f)' % (x.min(), newx.min()))
	if newx.max() > x.max():
		raise ValueError('x.max(%f) must be larger than newx.max(%f)' % (x.max(), newx.max()))

	return numpy.interp(newx, x, y)


def _regrid_spline(x, y, newx, allow_extrapolation=False):
	"""Implement regrid() function using spline fit and interpolation."""
	
	from scipy import interpolate

	if not allow_extrapolation:
		if newx.min() < x.min():
			raise ValueError('x.min(%f) must be smaller than newx.min(%f)' % (x.min(), newx.min()))
		if newx.max() > x.max():
			raise ValueError('x.max(%f) must be larger than newx.max(%f)' % (x.max(), newx.max()))
	spl = interpolate.splrep(x, y)
	newy = interpolate.splev(newx, spl)

	return newy
  

def downsample(vector, factor, rescale=True):
	"""Downsample (i.e. co-add consecutive numbers) a vector by an integer 
	factor.  Trims the input timeseries to be a multiple of the downsample 
	factor, if needed.  If rescale == True, then divides each sum by factor 
	to produce a mean value, otherwise just adds the values in the vector."""

	if (len(vector) % factor):
		_MATHUTIL_LOG.warning("Length of 'vector' is not divisible by 'factor'=%d, clipping!", factor)
		newlen = (len(vector)/factor)*factor
		_MATHUTIL_LOG.debug("Oldlen %d, newlen %d", len(vector), newlen)
		vector = vector[:newlen]
	if rescale:
		newvector = numpy.reshape(vector, (len(vector)/factor, factor))/float(factor)
	else:
		newvector = numpy.reshape(vector, (len(vector)/factor, factor))

	return numpy.add.reduce(newvector, 1)
    
    
def smooth(x,window_len=10,window='hanning'):
	"""Smooth the data using a window with requested size.  Stolen from SciPy 
	Cookbook at http://www.scipy.org/Cookbook/SignalSmooth
	
	This method is based on the convolution of a scaled window with the signal.
	The signal is prepared by introducing reflected copies of the signal 
	(with the window size) in both ends so that transient parts are minimized
	in the begining and end part of the output signal.
	
	Input:
	  * x: the input signal 
	  * window_len: the dimension of the smoothing window
	  * window: the type of window from 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'
	    flat window will produce a moving average smoothing.

	Output:
	  * the smoothed signal
		
	Example:
	  >>> from numpy import *
	  >>> t=linspace(-2,2,0.1)
	  >>> x=sin(t)+randn(len(t))*0.1
	  >>> y=smooth(x)
	
	.. seealso:: 
		numpy.hanning, numpy.hamming, numpy.bartlett, numpy.blackman, numpy.convolve
		scipy.signal.lfilter
	
	TODO: the window parameter could be the window itself if an array instead of a string"""

	if x.ndim != 1:
		raise ValueError("smooth only accepts 1 dimension arrays.")

	if x.size < window_len:
		raise ValueError("Input vector needs to be bigger than window size.")


	if window_len<3:
		return x


	if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
		raise ValueError("Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'")


	s=numpy.r_[2*x[0]-x[window_len:1:-1],x,2*x[-1]-x[-1:-window_len:-1]]
	#print(len(s))
	if window == 'flat': #moving average
		w=numpy.ones(window_len,'d')
	else:
		w=eval('numpy.'+window+'(window_len)')

	y=numpy.convolve(w/w.sum(),s,mode='same')
	return y[window_len-1:-window_len+1]


def cmagnitude(cmplx):
	"""Return the polar magnitudes of complex values."""
	
	return abs(cmplx)
    

def cphase(cmplx):
	"""Return the polar phases of complex values as radians.

	.. seealso::
		:func:`lsl.correlator.visUtils.argument`
	"""
	
	return numpy.angle(cmplx)
    
    
def cpolar(cmplx):
	"""Return the polar (magnitude, phase) representation of complex
	values (real, imaginary).  The return value is an array of shape (N,2),
	where N is the length of the cmplx input array."""
	
	if isinstance(cmplx, (numpy.ndarray, list, tuple)):
		return numpy.array(list(zip(cmagnitude(cmplx), cphase(cmplx))))
	else:
		return (cmagnitude(cmplx), cphase(cmplx))
    

def creal(cmplx):
	"""Return the real rectilinear component from complex values
	expressed in polar form (magnitude, phase)."""
	
	if isinstance(cmplx, numpy.ndarray):
		return (cmplx[...,0] * numpy.cos(cmplx[...,1]))
	else:
		return (cmplx[0] * math.cos(cmplx[1]))   

  
def cimag(cmplx):
	"""Return the imaginary rectilinear component from complex values
	expressed in polar form (magnitude, phase)."""
	
	if isinstance(cmplx, numpy.ndarray):
		return (cmplx[...,0] * numpy.sin(cmplx[...,1]))
	else:
		return (cmplx[0] * math.sin(cmplx[1]))    
    

def crect(cmplx):
	"""Return the rectilinear (real, imaginary) representation of complex
	values (magnitude, phase)."""
	
	if isinstance(cmplx, numpy.ndarray):
		ret = numpy.empty((len(cmplx),), numpy.complex_)
		ret.real = creal(cmplx)
		ret.imag = cimag(cmplx)
		return ret         
	else:
		return complex(creal(cmplx), cimag(cmplx))
        
    
def to_dB(factor):
	"""Convert from linear units to decibels."""
	
	return 10.0 * numpy.log10(factor)
    
    
def from_dB(dB):
	"""Convert from decibels to linear units."""
	
	return numpy.power(10.0, (dB/10.0))
    
    
def robustmean(arr):
	"""Take the robust mean of an array, normally a small section of a 
	spectrum, over which the mean can be assumed to be constant.  Makes two 
	passes discarding outliers >3 sigma ABOVE (not below) the mean.

	.. seealso::
		:func:`lsl.statistics.robust.robustMean`
	"""
	
	# First pass discarding points >3 sigma above mean
	mean = arr.mean()
	sd = arr.std()
	thresh = mean + 3.0*sd
	idx = numpy.where(arr < thresh)
	newarr = arr[idx]
	
	if len(newarr) == 0:
		# Warning, all points discarded.  Just return array mean
		_MATHUTIL_LOG.warning("All points discarded!, %f, %f", mean, sd)
		finalmean = mean
	else:
		# Second pass discarding points >3 sigma above mean
		newmean = newarr.mean()
		newsd = newarr.std()
		newthresh = newmean+3.0*newsd
		newidx = numpy.where(newarr < newthresh)
		finalarr = newarr[newidx]
		if len(finalarr) == 0:
			finalmean = newmean
		else:
			# Final mean of good points
			finalmean = finalarr.mean()

	return finalmean



# -*- coding: utf-8 -*-

"""Unit test for the lsl.misc.beamformer module."""

import os
import unittest
import numpy

from lsl.misc import beamformer
from lsl.common import stations as lwa_common


__revision__ = "$Revision:1 $"
__version__  = "0.1"
__author__    = "Jayce Dowell"


class beamformer_tests(unittest.TestCase):
	"""A unittest.TestCase collection of unit tests for the lsl.sim.dp
	module."""

	def test_calcDelay(self):
		"""Check that the beamformer.calcDelay function actually runs"""

		out = beamformer.calcDelay(numpy.array([1,2,3]))
		self.assertEqual(len(out), 3)
		
		out = beamformer.calcDelay(numpy.array([1,2,3]), freq=49.0e6)
		self.assertEqual(len(out), 3)

		out = beamformer.calcDelay(numpy.array([1,2,3]), freq=49.0e6, azimuth=45, elevation=30)
		self.assertEqual(len(out), 3)

	def test_pointing_limits(self):
		"""Test that beamformer.calcDelay respects the pointing limits"""

		# Azimuth  checks
		self.assertRaises(beamformer.BeamformingError, beamformer.calcDelay, numpy.array([1,2,3]), 49.0e6, -5, 30)
		self.assertRaises(beamformer.BeamformingError, beamformer.calcDelay, numpy.array([1,2,3]), 49.0e6, 365, 30)

		# Elevation checks
		self.assertRaises(beamformer.BeamformingError, beamformer.calcDelay, numpy.array([1,2,3]), 49.0e6, 45, -5)
		self.assertRaises(beamformer.BeamformingError, beamformer.calcDelay, numpy.array([1,2,3]), 49.0e6, 45, 95)

	def test_intDelayAndSum(self):
		"""Check that the beamformer.intDelayAndSum function actually runs"""
		
		stands = numpy.array([1,2,3])
		data = numpy.random.rand(3, 1000)
		
		beam = beamformer.intDelayAndSum(stands, data, azimuth=45.0, elevation=30.0)

	def test_intBeamShape(self):
		"""Check that the beamformer.intBeamShape function actually runs"""

		lwa1 = lwa_common.lwa1()
		stands = lwa1.getStands()

		out = beamformer.intBeamShape(stands, azimuth=135.0, elevation=60.0)
		i = out.argmax()
		az = i / 90
		el = i % 90
		self.assertAlmostEqual(az, 135.0, 1)
		self.assertAlmostEqual(el, 60.0, 1)
		

class  beamformer_test_suite(unittest.TestSuite):
	"""A unittest.TestSuite class which contains all of the lsl.sim.vis units 
	tests."""
	
	def __init__(self):
		unittest.TestSuite.__init__(self)
		
		loader = unittest.TestLoader()
		self.addTests(loader.loadTestsFromTestCase(beamformer_tests)) 


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

"""Unit test for the lsl.sim.dp module."""

import os
import unittest
import numpy
import tempfile

from lsl.sim import dp
from lsl.reader import tbw
from lsl.reader import tbn
from lsl.reader import drx
from lsl.common import dp as dp_common
from lsl.common import stations as lwa_common


__revision__ = "$Revision:1 $"
__version__  = "0.1"
__author__    = "Jayce Dowell"


class simdp_tests(unittest.TestCase):
	"""A unittest.TestCase collection of unit tests for the lsl.sim.dp
	module."""

	def setUp(self):
		"""Turn off all numpy warnings."""

		numpy.seterr(all='ignore')

	def test_basic_tbw(self):
		"""Test building a basic TBW signal"""

		testPath = tempfile.mkdtemp(prefix='simdp-', suffix='.tmp')
		testFile = os.path.join(testPath, 'tbw.dat')

		fh = open(testFile, 'wb')
		dp.basicSignal(fh, numpy.array([1,2,3,4]), 30000, mode='TBW', bits=12, tStart=1000)
		fh.close()

		# Check file size
		fileSize = os.path.getsize(testFile)
		nSamples = fileSize / tbw.FrameSize
		self.assertEqual(nSamples, 30000*4)

		# Check the time of the first frame
		fh = open(testFile, 'rb')
		frame = tbw.readFrame(fh)
		fh.close()
		self.assertEqual(frame.data.timeTag, 1000*dp_common.fS)

		# Check that the frames have the correct value of data bits
		self.assertEqual(frame.getDataBits(), 12)

		os.unlink(testFile)
		os.rmdir(testPath)

	def test_basic_tbn(self):
		"""Test building a basic TBN signal"""

		testPath = tempfile.mkdtemp(prefix='simdp-', suffix='.tmp')
		testFile = os.path.join(testPath, 'tbn.dat')

		fh = open(testFile, 'wb')
		dp.basicSignal(fh, numpy.array([1,2,3,4]), 2000, mode='TBN', filter=7, tStart=1000)
		fh.close()

		fileSize = os.path.getsize(testFile)
		nSamples = fileSize / tbn.FrameSize
		self.assertEqual(nSamples, 2000*4*2)

		fh = open(testFile, 'rb')
		frame = tbn.readFrame(fh)
		fh.close()
		self.assertEqual(frame.data.timeTag, 1000*dp_common.fS)

		os.unlink(testFile)
		os.rmdir(testPath)

	def test_basic_drx(self):
		"""Test building a basic DRX signal"""

		testPath = tempfile.mkdtemp(prefix='simdp-', suffix='.tmp')
		testFile = os.path.join(testPath, 'drx.dat')

		fh = open(testFile, 'wb')
		dp.basicSignal(fh, numpy.array([1,2,3,4]), 10, mode='DRX', filter=6, tStart=1000)
		fh.close()

		fileSize = os.path.getsize(testFile)
		nSamples = fileSize / drx.FrameSize
		self.assertEqual(nSamples, 10*4*2*2)

		fh = open(testFile, 'rb')
		frame = drx.readFrame(fh)
		fh.close()
		self.assertEqual(frame.data.timeTag, 1000*dp_common.fS)

		os.unlink(testFile)
		os.rmdir(testPath)


class  simdp_test_suite(unittest.TestSuite):
	"""A unittest.TestSuite class which contains all of the lsl.sim.vis units 
	tests."""
	
	def __init__(self):
		unittest.TestSuite.__init__(self)
		
		loader = unittest.TestLoader()
		self.addTests(loader.loadTestsFromTestCase(simdp_tests)) 


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

"""Unit test for lsl.skymap module."""

import unittest

from lsl import skymap, astro


__revision__  = "$Revision: 89 $"
__version__   = "0.1"
__author__    = "D.L.Wood"
__maintainer__ = "Jayce Dowell"


class skymap_tests(unittest.TestCase):
	"""A unittest.TestCase collection of unit tests for the lsl.skymap
	module."""
	
	def test_SkyMap_init(self):
		"""Test skymap.SkyMap class constructor method.""" 
		
		skymap.SkyMap()
		
	def test_SkyMap_ComputeTotalPowerFromSky(self):
		"""Test skymap.SkyMap.ComputeTotalPowerFromSky() method."""
		
		s = skymap.SkyMap()
		s.ComputeTotalPowerFromSky()
		
	def test_ProjectedSkyMap_init(self):
		"""Test skymap.ProjectedSkyMap constructor method."""
		
		s = skymap.SkyMap()
		skymap.ProjectedSkyMap(s, 20, 30, astro.get_julian_from_sys())
		
	def test_ProjectedSkyMap_ComputeTotalPowerFromVisibleSky(self):
		"""Test skymap.ProjectedSkyMap.ComputeTotalPowerFromVisibleSky() method."""
		
		s = skymap.SkyMap()
		p = skymap.ProjectedSkyMap(s, 20, 30, astro.get_julian_from_sys()) 
		p.ComputeTotalPowerFromVisibleSky()
		
	def test_ProjectedSkyMap_ComputeDirectionCosines(self):
		"""Test skymap.ProjectedSkyMap.ComputeDirectionCosines() method."""
		
		s = skymap.SkyMap()
		p = skymap.ProjectedSkyMap(s, 20, 30, astro.get_julian_from_sys())
		p.ComputeDirectionCosines()
        
        
class skymap_test_suite(unittest.TestSuite):
	"""A unittest.TestSuite class which contains all of the lwa_user.skymap
	module unit tests."""
	
	def __init__(self):
		unittest.TestSuite.__init__(self)
		
		loader = unittest.TestLoader()
		self.addTests(loader.loadTestsFromTestCase(skymap_tests))        
        
        
if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

"""Python module to handle the channelization and cross-correlation of TBW and
TBN data.  The main functions in this module are:
  * calcSpectra - calculate power spectra for a collection of signals
  * FXCorrelator - calculate cross power spectra for a collection of signals

Each function is set up to process the signals in parallel using the 
processing module and accepts a variety of options controlling the processing
of the data, including various window functions and time averaging.
"""

import os
import sys
import math
import numpy

from lsl.common import dp as dp_common
from lsl.common.constants import *
import uvUtils

__version__ = '0.2'
__revision__ = '$ Revision: 8 $'
__all__ = ['blackmanWindow', 'sincWindow', 'polyphaseFilter', 'calcSpectrum', 'calcSpectra', 'correlate', 'FXCorrelator', '__version__', '__revision__', '__all__']


def blackmanWindow(length):
	"""Generate a Blackman window of the specified length.  The window is returned
	as a numpy array."""

	if length > 1:
		N = length - 1
		x = numpy.linspace(0,length-1,length)
		w = 0.42 - 0.50*numpy.cos(2*numpy.pi*x/N) + 0.08*numpy.cos(4*numpy.pi*x/N)
	else:
		w = numpy.ones(1)

	return w


def sincWindow(length):
	"""Generate a sinc function of length elements such that the function 
	evaluates to zero at both ends of the range. The output is the sinc function
	as a numpy array."""

	x = numpy.arange(length)
	xPrime = 4*math.pi/float(length)*(x-length/2)
	fnc = numpy.sin(xPrime)/xPrime

	# Fix "the z=0 problem"
	z = numpy.where( xPrime == 0 )
	if len(z[0]) != 0:
		fnc[z[0]] = 1.0

	return fnc


def polyphaseFilter(signal, length=64, windows=4):
	"""Polyphase filter for the time series data.  This function multiplies the 
	input signal by a sinc function, breaks it into windows, and sum over the 
	different windows.  This results in a decimation of the signal by a factor 
	proportional to the number of windows used.  The output is the decimated 
	signal as a numpy array.

	.. warning::
		This function has not been tested and may not be implemented correctly.
	"""
	
	# The averaging length holds the size of the signal to be filtered
	aLength = length*windows
	filtered = numpy.zeros(signal.shape[0]/length/windows*length)

	# Loop over the signal
	for i in range(signal.shape[0]/aLength):
		# Current section of length 'length'
		s1 = signal[aLength*i:aLength*(i+1)]
		s1 = s1*sincWindow(aLength)

		# Create the 'windows' window components out of the sinc-multiplied signal
		parts = numpy.zeros((windows, length))
		for j in range(parts.shape[0]):
			parts[j,:] = s1[length*j:length*(j+1)]

		# Sum and save to the output array
		filtered[length*i:length*(i+1)] = parts.sum(axis=0)

	return filtered


def calcSpectrum(signal, LFFT=64, BlackmanFilter=False, PolyphaseFilter=False, verbose=False):
	"""Worker function for calcSpectra."""

	# Figure out if we are working with complex (I/Q) data or only real.  This
	# will determine how the FFTs are done since the real data mirrors the pos-
	# itive and negative Fourier frequencies.
	if signal.dtype.kind == 'c':
		lFactor = 1
	else:
		lFactor = 2

	out = numpy.zeros(LFFT-1)

	# Optionally apply a polyphase filter to the signals and store the results in
	# is intermediate signal variables 'is[12]'
	if PolyphaseFilter:
		intSignal = polyphaseFilter(signal, length=2*LFFT, windows=4)
		if verbose:
			print "  %i samples -> %i filtered samples" % (signal.shape[0], intSignal.shape[0])
	else:
		intSignal = signal

	# Remove the mean
	intSignal -= intSignal.mean()

	nChunks = intSignal.shape[0]/lFactor/LFFT
	if verbose:
		print "  %i samples -> %i chunks" % (intSignal.shape[0], nChunks)

	for i in range(nChunks):
		cs = intSignal[i*lFactor*LFFT:(i+1)*lFactor*LFFT]
		if BlackmanFilter:
			cs *= blackmanWindow(lFactor*LFFT)
		cf = numpy.fft.fft(cs, lFactor*LFFT)
		cp = numpy.abs(cf)**2.0
			
		out += cp[1:LFFT]
	out /= float(nChunks)

	return out


def calcSpectra(signals, LFFT=64, SampleAverage=None, BlackmanFilter=False, PolyphaseFilter=False, DisablePool=False, verbose=False, SampleRate=None, CentralFreq=0.0):
	"""Given a collection of time series data with inputs on the first dimension
	and data on the second, compute the spectra for all inputs.  By default, 
	all data in the time series are average into a single spectrum.  However, this 
	behavior can be modified if the SampleAverage keyword is set.  SampleAverage 
	specifies how many individual spectra of length LFFT are to be averaged."""

	# Figure out if we are working with complex (I/Q) data or only real.  This
	# will determine how the FFTs are done since the real data mirrors the pos-
	# itive and negative Fourier frequencies.
	if signals.dtype.kind == 'c':
		lFactor = 1
		doFFTShift = True
		CentralFreq = float(CentralFreq)
	else:
		lFactor = 2
		doFFTShift = False

	# Calculate the frequencies of the FFTs.  We do this for twice the FFT length
	# because the real-valued signal only occupies the positive part of the 
	# frequency space.
	if SampleRate is None:
		SampleRate = dp_common.fS
	freq = numpy.fft.fftfreq(lFactor*LFFT, d=1.0/SampleRate)
	# Deal with TBW and TBN data in the correct way
	if doFFTShift:
		freq += CentralFreq
		freq = numpy.fft.fftshift(freq)
	freq = freq[1:LFFT]

	# Calculate the number of output bins with respect to time.  If SampleAverage 
	# is None, average up everything.
	if SampleAverage is None:
		SampleAverage = signals.shape[1] / lFactor / LFFT
	nSamples = signals.shape[1] / lFactor / LFFT / int(SampleAverage)

	# The processing module allows for the creation of worker pools to help speed
	# things along.  If the processing module is found, use it.  Otherwise, set
	# the 'usePool' variable to false and run single threaded.
	try:
		from processing import Pool
		
		# To get results pack from the pool, you need to keep up with the workers.  
		# In addition, we need to keep up with which workers goes with which 
		# signal since the workers are called asychronisly.  Thus, we need a 
		# taskList array to hold tuples of signal ('count') and workers.
		taskPool = Pool(processes=6)
		taskList = []

		usePool = True
	except ImportError:
		usePool = False
	
	# Turn off the thread pool if we are explicitly told not to use it.
	if DisablePool:
		usePool = False

	output = numpy.zeros( (signals.shape[0], nSamples, freq.shape[0]) )
	for i in range(signals.shape[0]):
		for j in range(nSamples):
			# Figure out which section of each signal to average
			bchan = j*lFactor*LFFT*SampleAverage
			echan = (j+1)*lFactor*LFFT*SampleAverage
			if echan >= signals.shape[1]:
				echan = -1

			if verbose:
				print "Working on signal %i of %i, section %i to %i" % (i+1, signals.shape[0], bchan, echan)

			# If pool, pool...  Otherise don't
			if usePool:
				task = taskPool.apply_async(calcSpectrum, args=(signals[i,bchan:echan], ), 
									kwds={'LFFT': LFFT, 'BlackmanFilter': BlackmanFilter, 'PolyphaseFilter': PolyphaseFilter, 'verbose': verbose})
				taskList.append((i,j,task))
			else:
				tempPS = calcSpectrum(signals[i,bchan:echan], LFFT=LFFT, BlackmanFilter=BlackmanFilter, PolyphaseFilter=PolyphaseFilter, verbose=verbose)

				if doFFTShift:
					output[i,j,:] = numpy.fft.fftshift(tempPS)
				else:
					output[i,j,:] = tempPS

	# If pooling... Close the pool so that it knows that no ones else is joining.  
	# Then, join the workers together and wait on the last one to finish before 
	# saving the results.
	if usePool:
		taskPool.close()
		taskPool.join()

		# This is where he taskList list comes in handy.  We now know who did what
		# when we unpack the various results
		for i,j,task in taskList:
			if doFFTShift:
				output[i,j,:] = numpy.fft.fftshift(task.get())
			else:
				output[i,j,:] = task.get()

		# Destroy the taskPool
		del(taskPool)

	# Trim single dimensions from the output if they are found.  This is the 
	# default behavior
	output = numpy.squeeze(output)

	return (freq, output)


def correlate(signal1, signal2, stand1, stand2, LFFT=64, Overlap=1, BlackmanFilter=False, PolyphaseFilter=False, verbose=False, SampleRate=None, DlyCache=None, CentralFreq=0.0):
	"""Channalize and cross-correlate singal from two antennae.  Both the signals 
	and the stand numnbers are needed to implement time delay and phase corrections.
	The resulting visibilities from the cross-correlation are time average and a 
	LFFT-1 length numpy array is returned.  This array does not contain the DC 
	component of the signal."""

	import aipy

	# Figure out if we are working with complex (I/Q) data or only real.  This
	# will determine how the FFTs are done since the real data mirrors the pos-
	# itive and negative Fourier frequencies.
	if signal1.dtype.kind == 'c':
		lFactor = 1
		doFFTShift = True
		CentralFreq = float(CentralFreq)
	else:
		lFactor = 2
		doFFTShift = False

	# Calculate the frequencies of the FFTs.  We do this for twice the FFT length
	# because the real-valued signal only occupies the positive part of the 
	# frequency space.
	if SampleRate is None:
		SampleRate = dp_common.fS
	freq = numpy.fft.fftfreq(lFactor*LFFT, d=1.0/SampleRate)
	# Deal with TBW and TBN data in the correct way
	if doFFTShift:
		freq += CentralFreq
		freq = numpy.fft.fftshift(freq)
	freq = freq[1:LFFT]
	delayRef = len(freq)/2

	DlyCache.updateFreq(freq)

	# Calculate the frequency-based cable delays in seconds.
	if DlyCache is None:
		delay1 = uvUtils.signalDelay(stand1, freq=freq)
		delay2 = uvUtils.signalDelay(stand2, freq=freq)
	else:
		delay1 = DlyCache.signalDelay(stand1)
		delay2 = DlyCache.signalDelay(stand2)

	if delay2[delayRef] > delay1[delayRef]:
		delay1 = delay2 - delay1
		delay2 = delay2 - delay2
	else:
		delay2 = delay1 - delay2 
		delay1 = delay1 - delay1

	start1 = int(round(delay1[delayRef]*SampleRate))
	start2 = int(round(delay2[delayRef]*SampleRate))

	if verbose:
		print "  Delay for stands #%i: %.2f - %.2f ns (= %i samples)" % (stand1, delay1[-1]*1e9, delay1[0]*1e9, start1)
		print "  Delay for stands #%i: %.2f - %.2f ns (= %i samples)" % (stand2, delay2[-1]*1e9, delay2[0]*1e9, start2)

	# Optionally apply a polyphase filter to the signals and store the results in
	# is intermediate signal variables 'is[12]'
	if PolyphaseFilter:
		is1 = polyphaseFilter(signal1, length=lFactor*LFFT, windows=4)
		is2 = polyphaseFilter(signal2, length=lFactor*LFFT, windows=4)
	else:
		is1 = 1.0*signal1
		is2 = 1.0*signal2

	# Remove the mean
	is1 -= is1.mean()
	is2 -= is2.mean()

	# Compute the length of each time series and find the shortest one.
	length1 = (is1.shape[0] - start1) / LFFT / lFactor
	length2 = (is2.shape[0] - start2) / LFFT / lFactor
	# Factor in  the overlap amount for all but the last full sample
	length = numpy.array([length1, length2]).min() * Overlap - Overlap + 1

	# Begin computing the visibilities and loop over the signal chuncks in order 
	# to average in time.
	visibility = numpy.zeros(LFFT-1, dtype=numpy.complex_)
	for i in range(length):
		# Current chunks
		s1 = is1[(start1+int(lFactor*LFFT*(1.0*i)/Overlap)):(start1+int(lFactor*LFFT*((1.0*i)/Overlap+1)))]
		s2 = is2[(start2+int(lFactor*LFFT*(1.0*i)/Overlap)):(start2+int(lFactor*LFFT*((1.0*i)/Overlap+1)))]
		#import pylab
		#pylab.plot(s1.real)
		#pylab.plot(s2.real)
		#pylab.show()

		if BlackmanFilter:
			s1 *= blackmanWindow(lFactor*LFFT)
			s2 *= blackmanWindow(lFactor*LFFT)
		
		# The FFTs
		fft1 = numpy.fft.fft(s1, lFactor*LFFT)
		fft2 = numpy.fft.fft(s2, lFactor*LFFT)
		
		if doFFTShift:
			fft1 = numpy.fft.fftshift(fft1)
			fft2 = numpy.fft.fftshift(fft2)

		# Calculate the visibility of the desired part of the frequency space
		tempVis = fft1[1:LFFT]*(fft2[1:LFFT].conj())

		# Add it on to the master output visibility for averaging
		visibility += tempVis
	
	# Apply the phase rotator to the visibility
	visibility *= numpy.exp(2j*numpy.pi*freq*((delay1-start1/SampleRate)-(delay2-start2/SampleRate)))

	# Average and return
	visibility /= float(length)

	return visibility


def FXCorrelator(signals, stands, LFFT=64, Overlap=1, IncludeAuto=False, BlackmanFilter=False, PolyphaseFilter=False, CrossPol=None, DisablePool=False, verbose=False, SampleRate=None, CentralFreq=0.0):
	"""A basic FX correlators for the TBW data.  Given an 2-D array of signals
	(stands, time-series) and an array of stands, compute the cross-correlation of
	the data for all baselines.  If cross-polarizations need to be calculated, the
	CrossPol keyword allows for the other polarization data to be entered into the
	correlator.  Return the frequencies and visibilities as a two-elements tuple."""

	N = stands.shape[0]
	baselines = uvUtils.getBaselines(stands, IncludeAuto=IncludeAuto, Indicies=True)
	Nbase = len(baselines)
	output = numpy.zeros( (Nbase, LFFT-1), dtype=numpy.complex64)

	# Figure out if we are working with complex (I/Q) data or only real.  This
	# will determine how the FFTs are done since the real data mirrors the pos-
	# itive and negative Fourier frequencies.
	if signals.dtype.kind == 'c':
		lFactor = 1
		doFFTShift = True
		CentralFreq = float(CentralFreq)
	else:
		lFactor = 2
		doFFTShift = False

	if SampleRate is None:
		SampleRate = dp_common.fS
	freq = numpy.fft.fftfreq(lFactor*LFFT, d=1.0/SampleRate)
	if doFFTShift:
		freq += CentralFreq
		freq = numpy.fft.fftshift(freq)
	freq = freq[1:LFFT]

	# Define the cable/signal delay caches to help correlate along
	dlyCache = uvUtils.SignalCache(freq)

	# The processing module allows for the creation of worker pools to help speed
	# things along.  If the processing module is found, use it.  Otherwise, set
	# the 'usePool' variable to false and run single threaded.
	try:
		from processing import Pool
		
		# To get results pack from the pool, you need to keep up with the workers.  
		# In addition, we need to keep up with which workers goes with which 
		# baseline since the workers are called asychronisly.  Thus, we need a 
		# taskList array to hold tuples of baseline ('count') and workers.
		taskPool = Pool(processes=4)
		taskList = []

		usePool = True
	except ImportError:
		usePool = False

	# Turn off the thread pool if we are explicitly told not to use it.
	if DisablePool:
		usePool = False

	# Loop over baselines
	count = 0
	for i,j in baselines:
		if verbose:
			if i == j:
				print "Auto-correlating stand %i" % stands[i]
			else:
				print "Cross-correlating stands %i and %i" % (stands[i], stands[j])
			
		# If pool, pool...  Otherise don't
		if usePool:
			if CrossPol is not None:
				task = taskPool.apply_async(correlate, args=(signals[i,:], CrossPol[j,:], stands[i], stands[j]), 
								kwds={'LFFT': LFFT, 'Overlap': Overlap, 'BlackmanFilter': BlackmanFilter, 'PolyphaseFilter': PolyphaseFilter, 'verbose': verbose, 'SampleRate': SampleRate, 'DlyCache': dlyCache, 'CentralFreq': CentralFreq})
			else:
				task = taskPool.apply_async(correlate, args=(signals[i,:], signals[j,:], stands[i], stands[j]), 
								kwds={'LFFT': LFFT, 'Overlap': Overlap, 'BlackmanFilter': BlackmanFilter, 'PolyphaseFilter': PolyphaseFilter, 'verbose': verbose, 'SampleRate': SampleRate, 'DlyCache': dlyCache, 'CentralFreq': CentralFreq})
			taskList.append((count,task))
		else:
			if CrossPol is not None:
				tempCPS = correlate(signals[i,:], CrossPol[j,:], stands[i], stands[j], LFFT=LFFT, Overlap=Overlap, BlackmanFilter=BlackmanFilter, PolyphaseFilter=PolyphaseFilter, verbose=verbose, SampleRate=SampleRate, DlyCache=dlyCache, CentralFreq=CentralFreq)
			else:
				tempCPS = correlate(signals[i,:], signals[j,:], stands[i], stands[j], LFFT=LFFT, Overlap=Overlap, BlackmanFilter=BlackmanFilter,  PolyphaseFilter=PolyphaseFilter, verbose=verbose, SampleRate=SampleRate, DlyCache=dlyCache, CentralFreq=CentralFreq)

			if doFFTShift:
				output[count,:] = tempCPS
			else:
				output[count,:] = tempCPS
		count = count + 1

	# If pooling... Close the pool so that it knows that no ones else is joining.  
	# Then, join the workers together and wait on the last one to finish before 
	# saving the results.
	if usePool:
		taskPool.close()
		taskPool.join()

		# This is where he taskList list comes in handy.  We now know who did what
		# when we unpack the various results
		for count,task in taskList:
			if doFFTShift:
				output[count,:] = task.get()
			else:
				output[count,:] = task.get()

		# Destroy the taskPool
		del(taskPool)

	return (freq, output)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Example script for reading in DRX data and writing it to a SD-FITS file."""

import os
import sys
from lsl.reader import drx
from lsl.writer import sdfits

import matplotlib.pyplot as plt

def main(args):
	nSamples = os.path.getsize(args[0]) / drx.FrameSize
	print "Samples in file: ", nSamples
	fh = open(args[0], "rb", buffering=drx.FrameSize)
	nFpO = getFramesPerObs(fh)
	nBeams = getBeamCount(fh)
	print "Beams: ", nBeams
	print "Frames per Observations: ", nFpO
	blockBuffer = []
	blocks = []

	tStart = time.time()

	nSamples = (nSamples/4/16)*16

	fig = plt.figure()

	for i in range(0,nSamples):
		currBlock = readDRXBlock(fh)
		blockBuffer.append(currBlock)

		if len(blockBuffer) == 16:
			avgBlock = averageObservations(blockBuffer)
			#avgBlock = averageObservations2(blockBuffer, timeAvg=16, chanAvg=2)
			blocks.append(avgBlock)
			blockBuffer = []
	
	nChan = blocks[0].x1.data.iq.shape[0]
	outSpec = numpy.zeros((nSamples/16, nChan), dtype=numpy.complex64)
	outTime = numpy.zeros(nSamples/16)
	for row,block in zip(range(nSamples),blocks):
		outSpec[row,:] = block.x1.data.iq
		outTime[row] = block.x1.data.timeTag
	outSpec2 = numpy.zeros((nSamples/16, nChan), dtype=numpy.complex64)
	for row,block in zip(range(nSamples),blocks):
		outSpec2[row,:] = block.y1.data.iq
	outSpec3 = numpy.zeros((nSamples/16, nChan), dtype=numpy.complex64)
	for row,block in zip(range(nSamples),blocks):
		outSpec3[row,:] = block.x2.data.iq
	outSpec4 = numpy.zeros((nSamples/16, nChan), dtype=numpy.complex64)
	for row,block in zip(range(nSamples),blocks):
		outSpec4[row,:] = block.y2.data.iq

	tEnd = time.time()
	print 'Read %i frames in %0.3f s (%0.1f frames/s)' % (4*nSamples, (tEnd-tStart), 4*nSamples/(tEnd-tStart))
	
	writefits(outSpec, outTime)
	readfits('test-sdfits.fits')

	ax = fig.add_subplot(221)
	dB = outSpec - outSpec.mean(axis=0)
	dB = numpy.log10( (dB*dB.conj()).real )*10.0
	
	ax.imshow(numpy.transpose(dB), origin='lower')
	ax.set_title('Tuning 1, Pol. 0')
	ax.set_ylabel('Channel')
	ax.axis('auto')
	
	ax = fig.add_subplot(222)
	dB = outSpec2 - outSpec2.mean(axis=0)
	dB = numpy.log10( (dB*dB.conj()).real )*10.0
	
	ax.imshow(numpy.transpose(dB), origin='lower')
	ax.set_title('Tuning 1, Pol. 1')
	ax.axis('auto')

	ax = fig.add_subplot(223)
	dB = outSpec3 - outSpec3.mean(axis=0)
	dB = numpy.log10( (dB*dB.conj()).real )*10.0
	
	ax.imshow(numpy.transpose(dB), origin='lower')
	ax.set_title('Tuning 2, Pol. 0')
	ax.set_xlabel('Time')
	ax.set_ylabel('Channel')
	ax.axis('auto')

	ax = fig.add_subplot(224)
	dB = outSpec4 - outSpec4.mean(axis=0)
	dB = numpy.log10( (dB*dB.conj()).real )*10.0
	
	ax.imshow(numpy.transpose(dB), origin='lower')
	ax.set_title('Tuning 2, Pol. 1')
	ax.set_xlabel('Time')
	ax.axis('auto')

	plt.show()
	fig.savefig("readDRX.png")

	fh.close()

if __name__ == "__main__":
	main(sys.argv[1:])



# -*- coding: utf-8 -*-

"""Module that contains information about the LWA station sites.  This 
information includes:
  * latitude, 
  * longitude, 
  * elevation, and 
  * stand list.

The information for each site is stored as a LWAStation object.  This object 
has a variety of attributes for dealing with station-related parameters, such
as creating an ephem.Observer object for the station, getting the station's
geocentric location, and creating a geocentric coordinate transformation 
matrix for the station's baselines.

Currently, data exists for the following LWA stations:
  * lwa1 - LWA 1 near the VLA center.
"""

import math
import ephem
import numpy


__version__ = '0.2'
__revision__ = '$ Revision: 10 $'
__all__ = ['geo2ecef', 'LWAStation', 'lwa1', '__version__', '__revision__', '__all__']


def geo2ecef(lat, lon, elev):
	"""Convert latitude (rad), longitude (rad), elevation (m) to earth-
	centered, earth-fixed coordinates."""

	WGS84_a = 6378137.000000
	WGS84_b = 6356752.314245
	N = WGS84_a**2 / math.sqrt(WGS84_a**2*math.cos(lat)**2 + WGS84_b**2*math.sin(lon)**2)

	x = (N+elev)*math.cos(lat)*math.cos(lon)
	y = (N+elev)*math.cos(lat)*math.sin(lon)
	z = ((WGS84_b**2/WGS84_a**2)*N+elev)*math.sin(lat)

	return (x, y, z)


class LWAStation(object):
	"""Object to hold information about the a LWA station.  This object can
	create a ephem.Observer representation of itself and identify which stands
	were in use at a given time."""

	def __init__(self, name, lat, long, elev):
		self.name = name
		self.lat = lat*math.pi/180.0
		self.long = long*math.pi/180.0
		self.elev = elev

	def getObserver(self, date=None, JD=False):
		"""Return a ephem.Observer object for this site."""

		oo = ephem.Observer()
		oo.lat = self.lat
		oo.long = self.long
		oo.elev = self.elev
		oo.pressure = 0.0
		if date is not None:
			if JD:
				# If the date is Julian, convert to Dublin Julian Date 
				# which is used by ephem
				date -= 2415020.0
			oo.date = date

		return oo

	def getAIPYLocation(self):
		"""Return a tuple that can be used by AIPY for specifying a array
		location."""

		return (self.lat, self.long, self.elev)

	def getGeocentricLocation(self):
		"""Return a tuple with earth-centered, earth-fixed coordinates for the station."""

		return geo2ecef(self.lat, self.long, self.elev)

	def getECEFTransform(self):
		"""Return a 3x3 tranformation matrix that converts a baseline in 
		[east, north, elevation] to earh-centered, earth-fixed coordinates
		for that baseline [x, y, z].  Based off the 'local_to_eci' function
		in the lwda_fits-dev library."""

		return numpy.array([[0.0, -math.sin(self.lat), math.cos(self.lat)], 
						[1.0, 0.0,                 0.0], 
						[0.0, math.cos(self.lat), math.sin(self.lat)]])


class lwa1(LWAStation):
	"""Object to hold information about the first LWA station.  This object can
	create a ephem.Observer representation of itself and identify which stands
	were in use at a given time."""

	def __init__(self):
		super(lwa1, self).__init__('LWA-1', 34.070, -107.628, 2133.6)

	def getStands(self, date=None, JD=False):
		"""Return a numpy array that contains the stands used at a given 
		point in time."""

		oo = self.getObserver(date=date, JD=JD)
		fDate = float(oo.date)

		if fDate >= ephem.date('2010/11/23'):
			# Current as of 11/30/2010
			stands = numpy.array([206, 183, 174, 153, 38, 34, 181, 67, 80, 14, 118, 254, 246, 9, 168, 69, 258, 4, 205, 158])
		elif fDate >= ephem.Date('2010/10/29'):
			# Current as of 10/29/2010 for both TBN and TBW
			stands = numpy.array([258, 4, 158, 205, 246, 9, 69, 168, 80, 14, 254, 118, 38, 34, 67, 181, 206, 183, 153, 174])
		elif fDate >= ephem.Date('2010/09/15'):
			# Current as of 9/15/2010
			# Note:  This is for the input reversed TBW data.  Any TBN data should have 
			# the stand numbers exchanged in pairs.  For example, TBN data would be:
			# 258, 4, 158, 205, etc.
			stands = numpy.array([4, 258, 205, 158, 9, 246, 168, 69, 14, 80, 118, 254, 34, 38, 181, 67, 183, 206, 174, 153])
		elif fDate >= ephem.Date('2010/08/25'):
			# Between 8/25/2010 and 9/15/2010
			stands = numpy.array([214, 212, 228, 206, 127, 157, 187, 208, 123, 125])
		else:
			# Before 8/25/2010
			stands = numpy.array([214, 212, 228, 204, 225, 210, 187, 174, 123, 125])
		
		return stands


# -*- coding: utf-8 -*-

"""Unit test for lsl.common.paths module."""

import os
import unittest

from lsl.common import paths


__revision__ = "$Revision:1 $"
__version__  = "0.1"
__author__    = "Jayce Dowell"


class paths_tests(unittest.TestCase):
	"""A unittest.TestCase collection of unit tests for the lsl.common.paths tests
	module."""

	def test_module_path(self):
		"""Test the paths.module variable."""
		
		modPath = paths.moduleBuild

		astroFile = os.path.join(modPath, 'astro.py')
		self.assertTrue(os.path.exists(astroFile))

		tbwFile = os.path.join(modPath, 'reader', 'tbw.py')
		self.assertTrue(os.path.exists(tbwFile))

	def test_data_path(self):
		"""Test the paths.data variable."""
		
		dataPath = paths.dataBuild

		cablesFile = os.path.join(dataPath, 'lwa1-cables.csv')
		self.assertTrue(os.path.exists(cablesFile))

		timeFile = os.path.join(dataPath, 'astro', 'tai-utc.dat')
		self.assertTrue(os.path.exists(timeFile))

class paths_test_suite(unittest.TestSuite):
	"""A unittest.TestSuite class which contains all of the lsl.common.paths
	module unit tests."""
	
	def __init__(self):
		unittest.TestSuite.__init__(self)
		
		loader = unittest.TestLoader()
		self.addTests(loader.loadTestsFromTestCase(paths_tests)) 


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

"""This module contains a set of convenience functions to parse the output 
of NEC2, modify the input (.nec) file, and rerun NEC as necessary.
NEC2 is the Numerical Electromagnetics Code, developed at LLNL.
The version of NEC2 this code currently assumes is `here <http://www.physics.otago.ac.nz/research/electronics/nec/index.html>`_.
"""

from numpy import *
from lsl.misc.mathutil import regrid
import os
import re
import logging


__version__   = "0.1"
__revision__  = "$Revision: 92 $"
__all__ = ['CloseTo', 'open_and_get_nec_freq', 'change_nec_freq', 'calcIME', 'NECImpedance', 'NECPattern', 'whichNEC4', '__version__', '__revision__', '__all__']
__author__    = "P.S.Ray"
__maintainer__ = "Jayce Dowell"

_NEC_UTIL_LOG = logging.getLogger('nec_util')
_NEC_UTIL_LOG.setLevel(logging.INFO)


def CloseTo( x, y, epsilon=0.005 ):
	"""Return True if two numbers are within a specified fractional, not 
	absolute, tolerance of each other.  Tolerance epsilon is a keyword 
	parameter with a default of 0.005."""

	return ( 2.0*abs(x-y)/(x+y) < epsilon )


def open_and_get_nec_freq(fname):
	"""Open a NEC output file and return a tuple containing the open file 
	object and the first frequency found in the file (MHz)."""

	f = open(fname)

	# Start at beginning of file
	f.seek(0)

	# skip all lines until a line containing "STRUCTURE SPECIFICATION": this
	# effectively skips all comments (from CM cards) and text resulting from
	# reading Numerical Green's Function parts. Of course, if a user writes
	# "STRUCTURE SPECIFICATION" in his comment lines, this still fails...
	for line in f:
		if line.find('STRUCTURE SPECIFICATION') >= 0:
			break
	else:
		raise RuntimeError("STRUCTURE SPECIFICATION not found!")
		
	#  Now look for FREQUENCY and get the value
	for line in f:
		if line.find('FREQUENCY') >= 0:
			break
	for line in f:
		#print line
		if line.find('FREQUENCY=') >= 0:
			freq = float(line[line.find('=')+1:].split()[0])
			break
		if line.find('FREQUENCY :') >=0:
			freq = float(line[line.find(':')+1:].split()[0])
			break
	else:
		raise RuntimeError("Frequency value not found")
	
	_NEC_UTIL_LOG.debug("Found frequency %f MHz", freq)
	return (f, freq)


def change_nec_freq(necname, freq):
	"""Modify the FR card in a NEC input file to run at freq."""

	f = open(necname,'r+')
	buf = f.read()
	lines = buf.splitlines(True)
	# Substitute the freq in the right field of the FR card
	for i in range(len(lines)):
		if lines[i][:2] == 'FR':
			_NEC_UTIL_LOG.debug("Found line : %s", lines[i])
			vals = re.split(',| +',lines[i])
			_NEC_UTIL_LOG.debug("Vals = %s", vals)
			vals[5] = "%.2f" % freq
			lines[i] = " ".join(vals)
			# Make sure this line ends in newline
			if lines[i][-1] != '\n':
				lines[i] += '\n'
	# Rewrite the file
	f.seek(0)
	f.writelines(lines)
	f.truncate()
	f.close()


def calcIME(necname, myfreqs = None, zpre = 100):
	"""Compute the impedance mismatch efficiency (IME), for a given NEC run 
	and write out the results in a file.  Assumes a default preamplifier input 
	impedance of 100 ohms, unless overridden by the zpre keyword argument.
	Returns the frequencies calculated by NEC, unless myfreqs is set, in 
	which case it interpolates onto that frequency grid."""

	ant = NECImpedance(necname)
	gamma = (zpre - ant.z)/(zpre + ant.z)
	ime = (1 - abs(gamma)**2)
	if myfreqs is None:
		return (ant.freqs, ime)
	else:
		newime = regrid(ant.freqs, ime, myfreqs, method = 'linear')
		return (myfreqs, newime)


class NECImpedance:
	"""NECImpedance:
	Python class to read an array of impedance values from a NEC2 .out file The 
	.nec file should loop over a range of frequencies with an FR card like this:

		FR 0 91 0 0 10.0 1.0

	The RP card should be minimal to keep the runtime and output file size from 
	growing huge.  For example:

		RP 0,91,1,1000,0.,0.,1.0,1.0

	"""

	def __init__(self, necname):
		outname = os.path.splitext(necname)[0] + '.out'
		f = open(outname)

		# Start at beginning of file
		f.seek(0)
		
		# skip all lines until a line containing "STRUCTURE SPECIFICATION": this
		# effectively skips all comments (from CM cards) and text resulting from
		# reading Numerical Green's Function parts. Of course, if a user writes
		# "STRUCTURE SPECIFICATION" in his comment lines, this still fails...
		for line in f:
			if line.find('STRUCTURE SPECIFICATION') >= 0: break
		else:
			raise RuntimeError("STRUCTURE SPECIFICATION not found!")

		freqs = []
		impedances = []
		while (True):
			#  Now look for FREQUENCY and get the value
			for line in f:
				if line.find('FREQUENCY') >= 0:
					break
			for line in f:
				_NEC_UTIL_LOG.debug(line.strip())
				if line.find('FREQUENCY=') >= 0:
					freq = float(line[line.find('=')+1:].split()[0])
					break
				if line.find('FREQUENCY :') >=0:
					freq = float(line[line.find(':')+1:].split()[0])
					break
			else:
				_NEC_UTIL_LOG.debug("No more freqs...")
				break
			_NEC_UTIL_LOG.debug("Found frequency %f MHz",freq)
			for line in f:
				if line.find('ANTENNA INPUT PARAMETERS') >= 0:
					break
			gotimp = False
			for line in f:
				if line.find('IMPEDANCE') >= 0:
					gotimp = True
					break
			if not gotimp:
				raise RuntimeError("IMPEDANCE not found")
			for line in f:
				break
			for line in f:
				_NEC_UTIL_LOG.debug(line.strip())
				break

			# Here we need to add a space before - signs that
			# are not preceeded by an E, so it will parse
			line = re.sub(r'(\d)-',r'\1 -',line)
			re_z = float(line.split()[6])
			im_z = float(line.split()[7])
			freqs.append(freq)
			impedances.append(complex(re_z,im_z))
			
		self.freqs = array(freqs)
		self.z = array(impedances)


class NECPattern:
	"""NECPattern:
	Python class to read the pattern from a NEC2 .out file.  Note that the 
	.nec file should have an RP card to run over the full pattern, like this:

		RP 0,91,360,1000,0.,0.,1.0,1.0,0.

	The FR card should be a simple single frequency run:

		FR 0,1,0,0,74.0,1

	"""
        
	def __init__(self, necname, freq, rerun = True):

		# Modify NEC file to set FR card to use "freq"
		
		# Run NEC if necessary.
		# anntenna_pat_dB[az,alt] is the total gain in dB in the direction
		# az, alt (integer degrees), where
		# az (azimuth) runs from 0 to 359, where 0 is North
		# and alt (altitude) runs from 0 to 89 , where 0 is the horizon
		# The default pattern is all zeros (isotropic response)
		self.antenna_pat_dB=zeros(shape=(360,90),dtype=float_)

		outname = os.path.splitext(necname)[0] + '.out'
		try:
			f, filefreq = open_and_get_nec_freq(outname)
		except:
			print("NEC .out file not found! Running NEC")
			f = None
		
		if f is None or not CloseTo(filefreq,freq):
			if rerun:
				_NEC_UTIL_LOG.warning("NEC output file is at a different frequency \
					than the requested frequency: re-running")
				if f is not None:
					f.close()
				change_nec_freq(necname,freq)

				# Make sure we have NEC install
				if whichNEC4() is None:
					raise RuntimeError("NEC executable 'nec4d' not found in PATH")

				# Important NOTE:
				# This requires a modified version of NEC-4 that
				# takes 2 command line arguments instead of asking questions
				# interactively. See Paul Ray for info.
				cmdstr = "nec4d %s %s" % (necname, outname)
				ret=os.system(cmdstr)
				if ret != 0:
					raise RuntimeError("Bad return value from nec2++ call : %d" % ret)       
				f, filefreq = open_and_get_nec_freq(outname)
				if not CloseTo(filefreq,freq):
					raise ValueError("NEC failed to generate a file with the correct frequency.")
					
			else:
				raise ValueError("NEC output file is at a different frequency (%f) than the requested frequency (%f)." % \
					(filefreq, freq))
		
		#  Now look for RADIATION PATTERN and read it
		for line in f:
			if line.find('RADIATION PATTERN') >= 0:
				break
		else:
			raise RuntimeError("RADIATION PATTERN not found!")
					
		# Some versions of NEC2 output extraneous data after "RADIATION PATTERNS" before the actual data
		# and column labels (e.g. RANGE = and EXP (-JKR) values ).  Discard until
		# the true bottom of the column labels (look for DB) */
		for line in f:
			if line.find('DB') >= 0:
				break

		n = 0
		for line in f:
			cols = line.split()
			if len(cols) < 4:
				break
			# Parse theta and phi into ints so we can use for indexing
			# Convert theta from zenith angle to altitude
			theta = 90-int(cols[0].split('.')[0])
			phi = int(cols[1].split('.')[0])
			if theta < 0 or theta > 89 or phi > 359:
				#print "Skipping ",phi,theta
				continue
			powgain= float(cols[4])
			#print phi, theta, powgain
			self.antenna_pat_dB[phi,theta] = powgain
			n += 1
			_NEC_UTIL_LOG.debug("theta %d phi %d gain %f", theta,phi,powgain)


def whichNEC4():
	"""Return the path to the nec4d executable if it can be found in the 
	current path.  None otherwise.  This is useful for making sure that NEC
	is installed before trying to run something."""

	def is_exe(fpath):
		return os.path.exists(fpath) and os.access(fpath, os.X_OK)

	fpath, fname = os.path.split('nec4d')
	if fpath:
		if is_exe('nec4d'):
			return 'nec4d'
	else:
		for path in os.environ["PATH"].split(os.pathsep):
			exe_file = os.path.join(path, 'nec4d')
			if is_exe(exe_file):
				return exe_file

	return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Example script that reads in TBN data and runs a cross-correlation on it.  
The results are saved in the Miriad UV format."""

import os
import re
import sys
import aipy
import numpy
import getopt

import lsl.common.stations as lwa_common
import lsl.statistics.robust as robust
import lsl.reader.tbn as tbn
import lsl.reader.errors as errors
import lsl.correlator.uvUtils as uvUtils
import lsl.correlator.fx as fxc

import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter


def isValidDate(date):
	dateRE = re.compile(r'(?P<valid>\d{4}/\d{1,2}/\d{1,2}[T ]\d{1,2}:\d{1,2}:\d{1,2})')

	try:
		mtch = re.match(dateRE, date)
		mtch.group('valid')
		return True
	except:
		return False


def usage(exitCode=None):
	print """correlateTBN.py - cross-correlate data in a TBN file

Usage: correlateTBN.py [OPTIONS] file

Options:
-h --help             Display this help information
-c --central-freq     Central frequency of the observations in MHz
-d --date             Specify the date when the data were obtained
                      (YYYY/MM/DD HH:MM:SS)
-f --fft-length       Set FFT length (default = 512)
-t --avg-time         Window to average visibilities in time (seconds; 
                      default = 6 s)
-s --samples          Number of average visibilities to generate
                      (default = 10)
-q --quiet            Run correlateTBN in silent mode
"""

	if exitCode is not None:
		sys.exit(exitCode)
	else:
		return True


def parseConfig(args):
	config = {}
	# Command line flags - default values
	config['avgTime'] = 6
	config['LFFT'] = 512
	config['cFreq'] = 38.0e6
	config['date'] = '2010/01/01 00:00:00'
	config['samples'] = 10
	config['verbose'] = True
	config['args'] = []

	# Read in and process the command line flags
	try:
		opts, arg = getopt.getopt(args, "hqc:d:l:t:s:", ["help", "quiet", "central-freq=", "date=", "fft-length=", "avg-time=", "samples="])
	except getopt.GetoptError, err:
		# Print help information and exit:
		print str(err) # will print something like "option -a not recognized"
		usage(exitCode=2)
	
	# Work through opts
	for opt, value in opts:
		if opt in ('-h', '--help'):
			usage(exitCode=0)
		elif opt in ('-q', '--quiet'):
			config['verbose'] = False
		elif opt in ('-c', '--central-freq'):
			config['cFreq'] = float(value)*1e6
		elif opt in ('-d', '--date'):
			if not isValidDate(value):
				print "Invalid Date: %s" % value
				usage(exitCode=1)
			else:
				config['date'] = value
		elif opt in ('-l', '--fft-length'):
			config['LFFT'] = int(value)
		elif opt in ('-t', '--avg-time'):
			config['avgTime'] = int(value)
		elif opt in ('-s', '--samples'):
			config['samples'] = int(value)
		else:
			assert False
	
	# Add in arguments
	config['args'] = arg

	# Return configuration
	return config


def main(args):
	# Parse command line options
	config = parseConfig(args)
	filename = config['args'][0]

	# Length of the FFT
	LFFT = config['LFFT']

	# Setup the LWA station information and get the JD of observations
	lwa1 = lwa_common.lwa1()
	stands = lwa1.getStands(config['date'])
	lwa1_OO = lwa1.getObserver(date=config['date'], JD=False)
	jd = float(lwa1_OO.date) + 2415020.0

	fh = open(filename, "rb", buffering=tbn.FrameSize)
	test = tbn.readFrame(fh)
	if not test.header.isTBN():
		raise errors.notTBNError()
	fh.seek(0)
	nFpO = tbn.getFramesPerObs(fh)
	nFpO = nFpO[0] + nFpO[1]
	sampleRate = tbn.getSampleRate(fh, nFrames=nFpO)
	nInts = os.path.getsize(filename) / tbn.FrameSize / nFpO

	# Number of frames to read in at once and average
	nFrames = int(config['avgTime']*sampleRate/512)
	nSets = os.path.getsize(filename) / tbn.FrameSize / nFpO / nFrames

	print "TBN Data:  %s" % test.header.isTBN()
	print "Samples per observations: %i per pol." % (nFpO/2)
	print "Filter code is: %i" % tbn.getSampleRate(fh, nFrames=nFpO, FilterCode=True)
	print "Sampling rate is: %i Hz" % sampleRate
	print "Captures in file: %i (%.1f s)" % (nInts, nInts*512 / sampleRate)
	print "=="
	print "Station: %s" % lwa1.name
	print "Date observed: %s" % config['date']
	print "Julian day: %.5f" % jd
	print "Integration Time: %.3f s" % (512*nFrames/sampleRate)
	print "Number of averaged samples: %i" % nSets

	if config['samples'] > nSets:
		config['samples'] = nSets

	refTime = 0.0
	setTime = 0.0
	for s in range(config['samples']):
		count = {}
		masterCount = 0
		iTime = 0
		data = numpy.zeros((20,nFrames*512), dtype=numpy.complex64)
		for i in range(20*nFrames):
			# Read in the next frame and anticipate any problems that could occur
			try:
				cFrame = tbn.readFrame(fh, Verbose=False, SampleRate=sampleRate, CentralFreq=config['cFreq'], Gain=22)
			except errors.eofError:
				break
			except errors.syncError:
				print "WARNING: Mark 5C sync error on frame #%i" % (int(fh.tell())/TBWFrameSize-1)
				continue
			except errors.numpyError:
				break

			stand,pol = cFrame.header.parseID()
			aStand = 2*(stand-1)+pol
			if i == 0:
				if s == 0:
					refTime = 0
				setTime = s*512*nFrames/sampleRate
			if cFrame.header.frameCount % 500 == 0:
				print "%2i,%1i  %6.3f  %5i  %5i" % (stand, pol, cFrame.getTime(), cFrame.header.frameCount, cFrame.header.secondsCount)
			
			if aStand not in count.keys():
				count[aStand] = 0

			data[aStand,  count[aStand]*512:(count[aStand]+1)*512] = cFrame.data.iq
			
			count[aStand] = count[aStand] + 1
			masterCount = masterCount + 1

		print "Working on set #%i (%.3f seconds after set #1)" % ((s+1), (setTime-refTime))
		freqYY, outYY = fxc.FXCorrelator(data, stands, LFFT=LFFT, Overlap=1, IncludeAuto=True, verbose=True,  SampleRate=sampleRate, CentralFreq=config['cFreq'])
		uvw = uvUtils.computeUVW(stands, HA=((setTime-refTime)/3600.0), freq=freqYY, IncludeAuto=True)
		ccList = uvUtils.getBaselines(stands, IncludeAuto=True, Indicies=True)

		import pylab
		for i in [0, 1]:
			print freqYY.shape, outYY.shape, outYY[i,:].shape
			#pylab.plot(freqYY/1e6, numpy.log10(numpy.abs(outYY[i,:]))*10, label='%i' % i)
			pylab.plot(freqYY/1e6, numpy.angle(outYY[i,:]), label='%i' % i)
		pylab.legend(loc=0)
		pylab.show()

		toUse = numpy.where( (freqYY>10.0e6) & (freqYY<88.0e6) )
		toUse = toUse[0]

		if s == 0:
			uvoutname = os.path.basename(filename)
			uvoutname = uvoutname.replace('.dat', '.uv')
			if uvoutname.find('.uv') == -1:
				uvoutname = "%s.uv" % uvoutname

			uv = aipy.miriad.UV(uvoutname, status='new')
			tags = ['nants', 'nchan', 'npol', 'pol', 'sfreq', 'sdf', 'longitu', 'latitud', 'dec', 'ra', 'source']
			codes = ['i', 'i', 'i', 'i', 'r', 'r', 'r', 'r', 'r', 'r', 'a']
			for tag,code in zip(tags, codes):
				uv.add_var(tag, code)
			uv['source'] = 'zenith'
			uv['longitu'] = -107.628
			uv['latitud'] = 34.070
			uv['dec'] = 34.070
			uv['nants'] = stands.shape[0]
			uv['nchan'] = toUse.shape[0]
			uv['npol'] = 1
			uv['sfreq'] = freqYY[toUse[0]]
			uv['sdf'] = freqYY[toUse[1]]-freqYY[toUse[0]]

		uv['pol'] = -6
		for i in range(uvw.shape[0]):
			preamble = (numpy.squeeze(uvw[i,:,toUse[0]]), jd+(setTime-refTime)/3600.0/24.0, ccList[i])

			current = numpy.squeeze( outYY[i,toUse] )
			mask = numpy.zeros_like(current)

			uv.write(preamble, current, mask)
		del(data)

	fh.close()

if __name__ == "__main__":
	main(sys.argv[1:])
